import { Directive, ComponentFactoryResolver, Optional, ViewContainerRef, NgModule, Pipe, Injectable, ɵɵdefineInjectable, Component, Input, ɵɵinject, InjectionToken, ViewChild, ElementRef, EventEmitter, ChangeDetectorRef, Output, Inject, ViewChildren, ChangeDetectionStrategy, HostBinding, Type } from '@angular/core';
import { CommonModule } from '@angular/common';
import { RouterModule } from '@angular/router';
import { MatButtonModule } from '@angular/material/button';
import { MatToolbarModule } from '@angular/material/toolbar';
import { MatDialogRef, MatDialog, MatDialogModule, MAT_DIALOG_DATA, MatDialogConfig } from '@angular/material/dialog';
import { MatMenuModule, MatMenuTrigger } from '@angular/material/menu';
import { ReplaySubject, timer, of, Observable, merge } from 'rxjs';
import { map, take, timeout, last, first } from 'rxjs/operators';
import { AngularFirestore } from '@angular/fire/firestore';
import { FormsModule } from '@angular/forms';
import { MatCheckboxModule } from '@angular/material/checkbox';
import { MatIconModule } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { MatSelectModule } from '@angular/material/select';
import { MatSliderModule } from '@angular/material/slider';
import { MatSlideToggleModule } from '@angular/material/slide-toggle';
import { MatSnackBar, MatSnackBarModule } from '@angular/material/snack-bar';
import slugify from 'slugify';
import * as RandExp from 'randexp';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatCardModule } from '@angular/material/card';
import { MatExpansionPanel, MatExpansionModule } from '@angular/material/expansion';
import { MatChipsModule, MatIconModule as MatIconModule$1 } from '@angular/material';
import { MatListModule } from '@angular/material/list';
import { MatTooltipModule } from '@angular/material/tooltip';
import * as tinycolor_ from 'tinycolor2';
import { MatChipsModule as MatChipsModule$1 } from '@angular/material/chips';
import { MatRadioModule } from '@angular/material/radio';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiActionContextDirective {
    /**
     * @param {?} componentFactoryResolver
     * @param {?} dialogRef
     * @param {?} viewContainer
     */
    constructor(componentFactoryResolver, dialogRef, viewContainer) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.dialogRef = dialogRef;
        this.viewContainer = viewContainer;
    }
    /**
     * @return {?}
     */
    getContext() {
        return {
            componentFactoryResolver: this.componentFactoryResolver,
            dialogRef: this.dialogRef,
            viewContainer: this.viewContainer
        };
    }
}
CyberUiActionContextDirective.decorators = [
    { type: Directive, args: [{
                selector: '[cyberUiActionContext]',
                exportAs: 'cyberUiActionContext',
            },] }
];
/** @nocollapse */
CyberUiActionContextDirective.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: MatDialogRef, decorators: [{ type: Optional }] },
    { type: ViewContainerRef }
];
if (false) {
    /**
     * @type {?}
     * @private
     */
    CyberUiActionContextDirective.prototype.componentFactoryResolver;
    /**
     * @type {?}
     * @private
     */
    CyberUiActionContextDirective.prototype.dialogRef;
    /**
     * @type {?}
     * @private
     */
    CyberUiActionContextDirective.prototype.viewContainer;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiActionContextModule {
}
CyberUiActionContextModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    CyberUiActionContextDirective
                ],
                exports: [
                    CyberUiActionContextDirective,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Formats a duration (in milliseconds) as "minutes:seconds" timer
class TimerPipe {
    /**
     * @param {?} duration
     * @return {?}
     */
    transform(duration) {
        /** @type {?} */
        const minutes = Math.floor(duration / 1000 / 60);
        /** @type {?} */
        const seconds = (Math.floor(duration / 1000) % 60).toString().padStart(2, '0');
        return `${minutes}:${seconds}`;
    }
}
TimerPipe.decorators = [
    { type: Pipe, args: [{
                name: 'timer',
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiTimerPipeModule {
}
CyberUiTimerPipeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TimerPipe],
                exports: [TimerPipe],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A service for tracking timeboxes
// It's intended to run as a singleton and can handle multiple concurrent timeboxes
class CyberUiTimeboxService {
    constructor() {
        // Counter used to generate time box ids
        // Note: starts at 1 instead of 0 to avoid bugs around 0 being falsy
        this.idCounter = 1;
        // The currently active time boxes
        this.boxes = [];
        // Time box event stream. Used internally for recordkeeping.
        this.stream = new ReplaySubject(1);
        this.activeTimeboxes = new ReplaySubject(1);
        // Hook into the event stream to power the active time boxes replay subject
        this.stream.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        event => {
            this.activeTimeboxes.next({ boxes: this.boxes });
        }));
    }
    /**
     * @private
     * @return {?}
     */
    getNewTimeboxId() {
        // Increment the counter so that next time, it will return a different id
        return this.idCounter++;
    }
    /**
     * @private
     * @param {?} timebox
     * @return {?}
     */
    addTimebox(timebox) {
        this.boxes.push(timebox);
    }
    // Handle time box completion (called when the time box duration has elapsed)
    /**
     * @private
     * @param {?} timebox
     * @return {?}
     */
    onTimeboxComplete(timebox) {
        // Remove this time box from the boxes list
        this.boxes.splice(this.boxes.indexOf(timebox), 1);
        // Emit an end event to the stream
        this.stream.next({
            timeboxId: timebox.id,
            type: 'end',
        });
    }
    // Returns the current active time boxes, as an observable that will change over time
    // as the active time boxes change
    /**
     * @return {?}
     */
    getActiveTimeboxes() {
        return this.activeTimeboxes;
    }
    // Returns an observable counting down to 0 at the desired level of granularity
    // The observed quantity will be a duration in milliseconds
    /**
     * @param {?} options
     * @return {?}
     */
    getCountdown(options) {
        // Get the time box with the given id
        /** @type {?} */
        const timebox = this.getTimeboxById(options.timeboxId);
        // Initialize time remaining
        /** @type {?} */
        const timeRemaining = timebox.end - Date.now();
        // Determine the period to use
        /** @type {?} */
        const period = options.period || 1000;
        // Calculate how many updates will need to be made
        /** @type {?} */
        const numUpdates = Math.ceil(timeRemaining / period);
        return timer(0, period)
            // Return the number of ms remaining
            // The final emitted value might be < 0, so we take the max with 0
            // in order to return a final value of 0
            .pipe(map((/**
         * @param {?} i
         * @return {?}
         */
        (i) => Math.max(timeRemaining - period * i, 0))))
            .pipe(take(numUpdates));
    }
    /**
     * @param {?} id
     * @return {?}
     */
    getTimeboxById(id) {
        return this.boxes.find((/**
         * @param {?} timebox
         * @return {?}
         */
        timebox => timebox.id === id));
    }
    /**
     * @param {?} options
     * @return {?}
     */
    startTimebox(options) {
        /** @type {?} */
        const start = Date.now();
        /** @type {?} */
        const timebox = {
            id: this.getNewTimeboxId(),
            start: start,
            end: start + options.duration
        };
        timebox.timeoutId = window.setTimeout((/**
         * @return {?}
         */
        () => this.onTimeboxComplete(timebox)), options.duration);
        // Add this time box to the internal list of active time boxes
        this.addTimebox(timebox);
        // Emit a time box start event to the stream
        this.stream.next({
            timeboxId: timebox.id,
            type: 'start',
        });
        return {
            timeboxId: timebox.id,
        };
    }
    /**
     * @param {?} options
     * @return {?}
     */
    cancelTimebox(options) {
        /** @type {?} */
        const timebox = this.getTimeboxById(options.id);
        if (!timebox) {
            return {
                error: `Could not find time box with given id: ${options.id}`
            };
        }
        else {
            // Remove the scheduled end trigger
            clearTimeout(timebox.timeoutId);
            // Remove the time box from the list of active boxes
            this.boxes.splice(this.boxes.indexOf(timebox), 1);
            // Emit a cancel event to the stream
            this.stream.next({
                type: 'cancel',
                timeboxId: timebox.id
            });
            return {};
        }
    }
    /**
     * @param {?} options
     * @return {?}
     */
    addTimeToTimebox(options) {
        /** @type {?} */
        const timebox = this.getTimeboxById(options.id);
        if (!timebox) {
            return {
                error: `Could not find time box with given id: ${options.id}`
            };
        }
        else {
            // Remove the scheduled end trigger
            clearTimeout(timebox.timeoutId);
            // Calculate the new end time
            timebox.end += options.duration;
            // Set the new end trigger
            timebox.timeoutId = window.setTimeout((/**
             * @return {?}
             */
            () => this.onTimeboxComplete(timebox)), timebox.end - Date.now());
            // Emit a modify event to the stream
            this.stream.next({
                type: 'modify',
                timeboxId: timebox.id
            });
            return {};
        }
    }
}
CyberUiTimeboxService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root',
            },] }
];
/** @nocollapse */
CyberUiTimeboxService.ctorParameters = () => [];
/** @nocollapse */ CyberUiTimeboxService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiTimeboxService_Factory() { return new CyberUiTimeboxService(); }, token: CyberUiTimeboxService, providedIn: "root" });
if (false) {
    /**
     * @type {?}
     * @private
     */
    CyberUiTimeboxService.prototype.idCounter;
    /**
     * @type {?}
     * @private
     */
    CyberUiTimeboxService.prototype.boxes;
    /**
     * @type {?}
     * @private
     */
    CyberUiTimeboxService.prototype.stream;
    /**
     * @type {?}
     * @private
     */
    CyberUiTimeboxService.prototype.activeTimeboxes;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// An opinionated timebox countdown display component
// Strongly coupled to the cyber_ui timebox service implementation
class CyberUiTimeboxCountdownDisplayComponent {
    /**
     * @param {?} timeboxService
     */
    constructor(timeboxService) {
        this.timeboxService = timeboxService;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.timeboxId) {
            this.countdown = this.timeboxService.getCountdown({ timeboxId: changes.timeboxId.currentValue });
        }
    }
    /**
     * @return {?}
     */
    addFiveMinutes() {
        this.timeboxService.addTimeToTimebox({
            id: this.timeboxId,
            duration: 5 * 1000 * 60,
        });
        // Update the countdown
        this.countdown = this.timeboxService.getCountdown({ timeboxId: this.timeboxId });
    }
}
CyberUiTimeboxCountdownDisplayComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-timebox-countdown-display',
                template: "<button mat-button [matMenuTriggerFor]=\"menu\">{{countdown | async | timer}}</button>\n<mat-menu #menu=\"matMenu\">\n  <button mat-menu-item (click)=\"addFiveMinutes()\">Add 5 minutes</button>\n</mat-menu>"
            }] }
];
/** @nocollapse */
CyberUiTimeboxCountdownDisplayComponent.ctorParameters = () => [
    { type: CyberUiTimeboxService }
];
CyberUiTimeboxCountdownDisplayComponent.propDecorators = {
    timeboxId: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiTimeboxCountdownDisplayComponent.prototype.timeboxId;
    /** @type {?} */
    CyberUiTimeboxCountdownDisplayComponent.prototype.countdown;
    /** @type {?} */
    CyberUiTimeboxCountdownDisplayComponent.prototype.timeboxService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiTimeboxCountdownDisplayModule {
}
CyberUiTimeboxCountdownDisplayModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatButtonModule,
                    MatMenuModule,
                    CyberUiTimerPipeModule,
                ],
                declarations: [
                    CyberUiTimeboxCountdownDisplayComponent,
                ],
                exports: [
                    CyberUiTimeboxCountdownDisplayComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiAttentionalAgendaItem {
    /**
     * @param {?} options
     * @param {?} service
     */
    constructor(options, service) {
        this.service = service;
        this.label = options.label;
        this.labelRouterLink = options.labelRouterLink;
        this.added = this.getAdded(options);
        this.timeboxId = this.getTimeboxId(options);
        this.actions = this.getActions(options);
    }
    // Clears this item from the agenda
    /**
     * @return {?}
     */
    clear() {
        this.service.clearItem(this);
    }
    // Returns the value to use for the `added` timestamp
    /**
     * @private
     * @param {?} options
     * @return {?}
     */
    getAdded(options) {
        return options.added || Date.now();
    }
    /**
     * @private
     * @param {?} options
     * @return {?}
     */
    getTimeboxId(options) {
        if (options.timeboxId)
            return options.timeboxId;
        else if (options.timeboxDuration) {
            return this.service.timeboxService.startTimebox({
                duration: options.timeboxDuration
            }).timeboxId;
        }
    }
    /**
     * @private
     * @param {?} options
     * @return {?}
     */
    getActions(options) {
        return options.actions || of([]);
    }
}
if (false) {
    /** @type {?} */
    CyberUiAttentionalAgendaItem.prototype.label;
    /** @type {?} */
    CyberUiAttentionalAgendaItem.prototype.labelRouterLink;
    /** @type {?} */
    CyberUiAttentionalAgendaItem.prototype.added;
    /** @type {?} */
    CyberUiAttentionalAgendaItem.prototype.timeboxId;
    /** @type {?} */
    CyberUiAttentionalAgendaItem.prototype.actions;
    /** @type {?} */
    CyberUiAttentionalAgendaItem.prototype.service;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiAttentionalAgendaItemDisplayComponent {
}
CyberUiAttentionalAgendaItemDisplayComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-attentional-agenda-item-display',
                template: "<mat-toolbar>\n  <cyber-ui-timebox-countdown-display\n    *ngIf=\"item.timeboxId\"\n    [timeboxId]=\"item.timeboxId\">\n  </cyber-ui-timebox-countdown-display>\n  <span class=\"label\"\n        [class.clickable]=\"item.labelRouterLink !== undefined\"\n        [routerLink]=\"item.labelRouterLink\"\n        >{{item.label}}</span>\n  <button\n      mat-button\n      *ngFor=\"let action of item.actions | async\"\n      cyberUiActionContext\n      #actionContext=\"cyberUiActionContext\"\n      (click)=\"action.handler(actionContext.getContext())\">\n    {{action.label}}\n  </button>\n  <button mat-button (click)=\"item.clear()\">CLEAR</button>\n</mat-toolbar>\n",
                styles: [".label{flex:1;font-size:15px;overflow:hidden;text-overflow:ellipsis}.label.clickable{cursor:pointer}"]
            }] }
];
CyberUiAttentionalAgendaItemDisplayComponent.propDecorators = {
    item: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiAttentionalAgendaItemDisplayComponent.prototype.item;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiAttentionalAgendaItemDisplayModule {
}
CyberUiAttentionalAgendaItemDisplayModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    RouterModule,
                    CyberUiActionContextModule,
                    CyberUiTimeboxCountdownDisplayModule,
                    MatButtonModule,
                    MatToolbarModule,
                ],
                declarations: [
                    CyberUiAttentionalAgendaItemDisplayComponent,
                ],
                exports: [
                    CyberUiAttentionalAgendaItemDisplayComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiAttentionalAgendaSnapshot {
    /**
     * @param {?} options
     */
    constructor(options) {
        this.items = options.items;
    }
    // Returns the item which was most recently added to the agenda
    /**
     * @return {?}
     */
    getMostRecentlyAddedItem() {
        return this.items.reduce((/**
         * @param {?} mostRecentItem
         * @param {?} candidateItem
         * @return {?}
         */
        (mostRecentItem, candidateItem) => {
            if (mostRecentItem && mostRecentItem.added > candidateItem.added) {
                return mostRecentItem;
            }
            else {
                return candidateItem;
            }
        }));
    }
}
if (false) {
    /** @type {?} */
    CyberUiAttentionalAgendaSnapshot.prototype.items;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiAttentionalAgendaDisplayComponent {
}
CyberUiAttentionalAgendaDisplayComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-attentional-agenda-display',
                template: "<ng-container *ngIf=\"agenda\">\n  <cyber-ui-attentional-agenda-item-display\n    *ngFor=\"let item of agenda.items\"\n    [item]=\"item\">\n  </cyber-ui-attentional-agenda-item-display>\n</ng-container>"
            }] }
];
CyberUiAttentionalAgendaDisplayComponent.propDecorators = {
    agenda: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiAttentionalAgendaDisplayComponent.prototype.agenda;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiAttentionalAgendaDisplayModule {
}
CyberUiAttentionalAgendaDisplayModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    CyberUiAttentionalAgendaItemDisplayModule,
                ],
                declarations: [
                    CyberUiAttentionalAgendaDisplayComponent,
                ],
                exports: [
                    CyberUiAttentionalAgendaDisplayComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A service for managing subjects of the user's attention
class CyberUiAttentionalAgendaService {
    /**
     * @param {?} timeboxService
     */
    constructor(timeboxService) {
        this.timeboxService = timeboxService;
        this.state = new ReplaySubject(1);
        this.items = [];
    }
    /**
     * @param {?} options
     * @return {?}
     */
    addItemToAttentionalAgenda(options) {
        /** @type {?} */
        const newItem = new CyberUiAttentionalAgendaItem(options, this);
        this.items.push(newItem);
        // Emit a new state for consumers
        this.emitNewState();
    }
    /**
     * @param {?} item
     * @return {?}
     */
    clearItem(item) {
        // Remove the item from the agenda
        this.items.splice(this.items.indexOf(item), 1);
        // Emit a new state for consumers
        this.emitNewState();
    }
    /**
     * @private
     * @return {?}
     */
    emitNewState() {
        /** @type {?} */
        const newState = new CyberUiAttentionalAgendaSnapshot({
            items: this.items
        });
        this.state.next(newState);
    }
    /**
     * @return {?}
     */
    getState() {
        return this.state;
    }
}
CyberUiAttentionalAgendaService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
CyberUiAttentionalAgendaService.ctorParameters = () => [
    { type: CyberUiTimeboxService }
];
/** @nocollapse */ CyberUiAttentionalAgendaService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiAttentionalAgendaService_Factory() { return new CyberUiAttentionalAgendaService(ɵɵinject(CyberUiTimeboxService)); }, token: CyberUiAttentionalAgendaService, providedIn: "root" });
if (false) {
    /**
     * @type {?}
     * @private
     */
    CyberUiAttentionalAgendaService.prototype.state;
    /**
     * @type {?}
     * @private
     */
    CyberUiAttentionalAgendaService.prototype.items;
    /** @type {?} */
    CyberUiAttentionalAgendaService.prototype.timeboxService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DEFAULT_COLLECTION_ID = 'things';
// Consuming applications may elect to provide their own implementation of this
class CyberUiFirestorePathService {
    // Returns a suitable path to the document for this model, or undefined
    /**
     * @param {?} model
     * @param {?=} saveOptions
     * @return {?}
     */
    calculatePath(model, saveOptions = {}) {
        /** @type {?} */
        const collectionId = this.calculateCollectionId(model, saveOptions);
        if (!collectionId) {
            console.error('Could not determine collectionId');
            return undefined;
        }
        else if (!model.data.id) {
            console.error('Could not determine model id');
            return undefined;
        }
        else {
            return `${collectionId}/${model.data.id}`;
        }
    }
    // Returns the collection in which to save a given model, or undefined
    // Undefined is returned if the collection id cannot be determined and
    // the model should not be saved
    /**
     * @private
     * @param {?} model
     * @param {?} saveOptions
     * @return {?}
     */
    calculateCollectionId(model, saveOptions) {
        return saveOptions.collectionId || model.collectionId || DEFAULT_COLLECTION_ID;
    }
}
CyberUiFirestorePathService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */ CyberUiFirestorePathService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiFirestorePathService_Factory() { return new CyberUiFirestorePathService(); }, token: CyberUiFirestorePathService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A reference implementation for persisting data in Firestore
class CyberUiFirestoreBackend {
    /**
     * @param {?} firestore
     * @param {?} firestorePathService
     */
    constructor(firestore, firestorePathService) {
        this.firestore = firestore;
        this.firestorePathService = firestorePathService;
    }
    // Deletes the Firestore document corresponding to this model
    // Note: this is an unsafe, permanent operation (when successful)
    // It is recommended to user CyberUiFirestoreBackedModelService#deleteModel
    // in order to place deletion behind a user-facing confirmation dialog
    /**
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    deleteModel(model, options = {}) {
        /** @type {?} */
        const path = this.firestorePathService.calculatePath(model);
        // Refuse to delete a model whose path cannot be determined
        if (!path) {
            return Promise.reject('Backend refuses to delete a model whose path cannot be determined');
        }
        /** @type {?} */
        const doc = this.firestore.doc(path);
        return doc.delete();
    }
    // Save this model as a Firestore document
    /**
     * @param {?} model
     * @param {?=} options
     * @return {?}
     */
    saveModel(model, options = {}) {
        // Assign the model a document id if it doesn't already have one
        model.data.id = model.data.id || this.firestore.createId();
        // Calculate the path at which to save the document
        /** @type {?} */
        const path = this.firestorePathService.calculatePath(model, options);
        // Refuse to save if the path could not be determined
        if (!path) {
            return Promise.reject('Failed to calculate path for document');
        }
        // Get a reference to the Firestore document
        /** @type {?} */
        const doc = this.firestore.doc(path);
        return doc.set(model.data);
    }
}
CyberUiFirestoreBackend.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
CyberUiFirestoreBackend.ctorParameters = () => [
    { type: AngularFirestore },
    { type: CyberUiFirestorePathService }
];
/** @nocollapse */ CyberUiFirestoreBackend.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiFirestoreBackend_Factory() { return new CyberUiFirestoreBackend(ɵɵinject(AngularFirestore), ɵɵinject(CyberUiFirestorePathService)); }, token: CyberUiFirestoreBackend, providedIn: "root" });
if (false) {
    /** @type {?} */
    CyberUiFirestoreBackend.prototype.firestore;
    /** @type {?} */
    CyberUiFirestoreBackend.prototype.firestorePathService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Q. Why do we need a model in the first place?
// A. It would be really nice to be able to write code such as model.save().then(...)
// A. It's nice to not need a reference to the underlying service from the template logic
//
// Q. Should the model keep it's own internal data object for saving to Firestore,
//    OR should it just store the properties on itself,
//    OR should consumers have to pass in a data object to use as the state?
// A. For now, let's have consumers pass in a data object to use as the state
//    There hasn't been a strong enough argument yet for making a separate state copy
class CyberUiFirestoreBackedModel {
    /**
     * @param {?} firestoreBackedModelService
     * @param {?=} data
     */
    constructor(firestoreBackedModelService, data = {}) {
        this.firestoreBackedModelService = firestoreBackedModelService;
        this.data = data;
    }
    // Shows an edit dialog corresponding to this model
    /**
     * @return {?}
     */
    editInDialog() {
        // TODO Implement using EditDialogService
    }
    /**
     * @param {?} propertyName
     * @return {?}
     */
    getProperty(propertyName) {
        return this.data[propertyName];
    }
    /**
     * @param {?} propertyName
     * @param {?} value
     * @return {?}
     */
    setProperty(propertyName, value) {
        this.data[propertyName] = value;
    }
    // Save this model to the backend
    /**
     * @param {?=} options
     * @return {?}
     */
    save(options = {}) {
        return this.firestoreBackedModelService.saveModel(this, options);
    }
    // Deletes the model from the backend, pending a confirmation from the user
    /**
     * @param {?} options
     * @return {?}
     */
    delete(options) {
        return this.firestoreBackedModelService.deleteModel(this, options);
    }
    // TODO Address security implications of this; maybe make a copy?
    /**
     * @return {?}
     */
    toJson() {
        return this.data;
    }
}
if (false) {
    /** @type {?} */
    CyberUiFirestoreBackedModel.prototype.collectionId;
    /** @type {?} */
    CyberUiFirestoreBackedModel.prototype.id;
    /** @type {?} */
    CyberUiFirestoreBackedModel.prototype.firestoreBackedModelService;
    /** @type {?} */
    CyberUiFirestoreBackedModel.prototype.data;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Since models are frequently constructed in Cyber UI,
// this service exists to group all of the various needed dependencies
// into a single injectable service
// That way, we don't have to constantly change the model constructors
// when adding or removing depdencies
class CyberUiFirestoreBackedModelService {
    /**
     * @param {?} backend
     */
    constructor(backend) {
        this.backend = backend;
    }
    /**
     * @return {?}
     */
    getNewEmptyModel() {
        return new CyberUiFirestoreBackedModel(this, {});
    }
    /**
     * @param {?} model
     * @param {?} options
     * @return {?}
     */
    deleteModel(model, options) {
        // Confirm the deletion in all cases except when options.confirm is false
        /** @type {?} */
        const confirm = options.confirm !== false;
        if (confirm) {
            // TODO Add a confirmation dialog step here
        }
        else {
        }
        // TODO For now we are just always deleting the model. This needs to be fixed.
        return this.backend.deleteModel(model, options);
    }
    /**
     * @param {?} model
     * @param {?} options
     * @return {?}
     */
    saveModel(model, options) {
        // Refuse to save an empty model to the backend, unless options.allowEmptySave is true
        if (!options.allowEmptySave) {
            if (Object.keys(model.data).length === 0) {
                return Promise.reject('Model refuses to be saved when it is empty');
            }
        }
        // Set the 'added' date on the model, unless options specify otherwise or the property is already set
        // TODO Add tests for this functionality
        if (options.setAddedIfUndefined === undefined || options.setAddedIfUndefined === true) {
            if (model.data.added === undefined) {
                model.data.added = Date.now();
            }
        }
        return this.backend.saveModel(model, options);
    }
}
CyberUiFirestoreBackedModelService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
CyberUiFirestoreBackedModelService.ctorParameters = () => [
    { type: CyberUiFirestoreBackend }
];
/** @nocollapse */ CyberUiFirestoreBackedModelService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiFirestoreBackedModelService_Factory() { return new CyberUiFirestoreBackedModelService(ɵɵinject(CyberUiFirestoreBackend)); }, token: CyberUiFirestoreBackedModelService, providedIn: "root" });
if (false) {
    /** @type {?} */
    CyberUiFirestoreBackedModelService.prototype.backend;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiFirestoreBackendModule {
}
CyberUiFirestoreBackendModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    CyberUiFirestoreBackend,
                    CyberUiFirestoreBackedModelService,
                    CyberUiFirestorePathService,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const CYBER_UI_MODEL_SERVICE = new InjectionToken('CyberUiModelService');
/**
 * @record
 */
function CyberUiModelService() { }
if (false) {
    /**
     * @return {?}
     */
    CyberUiModelService.prototype.getNewEmptyModel = function () { };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A non-persistent fake model to use in tests and demos
class FakeModel {
    constructor() {
        this.data = {}; // the data object in which the model's data will be stored
    }
    // the data object in which the model's data will be stored
    /**
     * @param {?} propertyName
     * @return {?}
     */
    getProperty(propertyName) {
        return this.data[propertyName];
    }
    /**
     * @param {?} propertyName
     * @param {?} value
     * @return {?}
     */
    setProperty(propertyName, value) {
        this.data[propertyName] = value;
    }
    /**
     * @return {?}
     */
    isEmpty() {
        return Object.getOwnPropertyNames(this.data).length === 0;
    }
    /**
     * @return {?}
     */
    save() {
        if (this.isEmpty()) {
            return Promise.reject('FakeModel refuses to be saved when it is empty');
        }
        else {
            return Promise.resolve();
        }
    }
    /**
     * @return {?}
     */
    toJson() {
        return this.data;
    }
}
if (false) {
    /** @type {?} */
    FakeModel.prototype.data;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FakeModelService {
    /**
     * @return {?}
     */
    getNewEmptyModel() {
        return new FakeModel();
    }
}
FakeModelService.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiFakeBackendModule {
}
CyberUiFakeBackendModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    { provide: CYBER_UI_MODEL_SERVICE, useClass: FakeModelService },
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A service for managing and appending to a delegatable thing's delegation history
class CyberUiDelegationHistoryService {
    /**
     * @param {?} thing
     * @param {?} actionRecord
     * @return {?}
     */
    logDelegationAction(thing, actionRecord) {
        // Initialize the 'delegationHistory' property if it's not yet defined
        if (thing.delegationHistory === undefined) {
            thing.delegationHistory = [];
        }
        // Push the latest record onto the history
        thing.delegationHistory.push(actionRecord);
    }
}
CyberUiDelegationHistoryService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ CyberUiDelegationHistoryService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiDelegationHistoryService_Factory() { return new CyberUiDelegationHistoryService(); }, token: CyberUiDelegationHistoryService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO Implement this and achieve a reasonable UI for it
class DelegationHistoryField {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiDelegationMenuService {
    constructor() {
        // The latest state to use to power the delegation menu
        this.state = new ReplaySubject(1);
        this.registeredDelegationTargets = [];
    }
    // Registers a delegation target with the service
    // At the moment, this is irrevocable, but it doesn't need to be
    /**
     * @param {?} options
     * @return {?}
     */
    registerDelegationTargets(options) {
        if (options.clearPreviousRegistrations) {
            this.registeredDelegationTargets = options.targets.slice();
        }
        else {
            this.registeredDelegationTargets = this.registeredDelegationTargets.concat(options.targets);
        }
        // Send any subscribed delegation menus the latest menu state
        this.state.next({
            // Provide a shallow copy of registeredDelegationTargets in order to
            // prevent subscribers from altering this protected internal state
            delegationTargets: this.registeredDelegationTargets.slice()
        });
    }
}
CyberUiDelegationMenuService.decorators = [
    { type: Injectable }
];
if (false) {
    /** @type {?} */
    CyberUiDelegationMenuService.prototype.state;
    /**
     * @type {?}
     * @private
     */
    CyberUiDelegationMenuService.prototype.registeredDelegationTargets;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DelegationMenuComponent {
    /**
     * @param {?} delegationMenuService
     */
    constructor(delegationMenuService) {
        this.delegationMenuService = delegationMenuService;
        // All applicable state is stored in the DelegationMenuState
        this.state = {
            delegationTargets: []
        };
        // Subscribe to state updates from the delegation menu service
        this.delegationMenuServiceStateSubscription =
            delegationMenuService.state.subscribe((/**
             * @param {?} state
             * @return {?}
             */
            state => this.state = state));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        // Release the subscription in order to avoid a memory leak
        this.delegationMenuServiceStateSubscription.unsubscribe();
    }
}
DelegationMenuComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-delegation-menu',
                template: "<span #trigger [matMenuTriggerFor]=\"menu\"></span>\n<mat-menu #menu=\"matMenu\">\n  <div mat-menu-item *ngIf=\"state.delegationTargets.length === 0\" disabled=\"true\">No delegation targets registered</div>\n  <button\n      *ngFor=\"let delegationTarget of state.delegationTargets\"\n      mat-menu-item\n      cyberUiActionContext\n      #actionContext=\"cyberUiActionContext\"\n      (click)=\"delegationTarget.delegationTargetHandler(delegatableThing, actionContext)\">{{delegationTarget.label}}</button>\n</mat-menu>\n"
            }] }
];
/** @nocollapse */
DelegationMenuComponent.ctorParameters = () => [
    { type: CyberUiDelegationMenuService }
];
DelegationMenuComponent.propDecorators = {
    delegatableThing: [{ type: Input }],
    trigger: [{ type: ViewChild, args: ['trigger', { read: MatMenuTrigger, static: true },] }]
};
if (false) {
    /** @type {?} */
    DelegationMenuComponent.prototype.delegatableThing;
    /** @type {?} */
    DelegationMenuComponent.prototype.trigger;
    /** @type {?} */
    DelegationMenuComponent.prototype.state;
    /** @type {?} */
    DelegationMenuComponent.prototype.delegationMenuServiceStateSubscription;
    /** @type {?} */
    DelegationMenuComponent.prototype.delegationMenuService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiDelegationMenuModule {
}
CyberUiDelegationMenuModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatMenuModule,
                    CyberUiActionContextModule
                ],
                declarations: [DelegationMenuComponent],
                exports: [DelegationMenuComponent],
                providers: [CyberUiDelegationMenuService],
                entryComponents: [DelegationMenuComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A utility method for mounting and triggering a delegation menu in response to a task action
// (such as a 'DELEGATE TO...' button being selected)
/**
 * @param {?} delegatableThing
 * @param {?} ctx
 * @return {?}
 */
function delegationMenuActivationHandler(
// The thing to be delegated
delegatableThing, 
// The display context for the menu
ctx) {
    // Mount and activate the delegation menu
    // First, clear previously mounted components
    ctx.viewContainer.clear();
    /** @type {?} */
    const factory = ctx.componentFactoryResolver.resolveComponentFactory(DelegationMenuComponent);
    /** @type {?} */
    const delegationMenuCmpRef = ctx.viewContainer.createComponent(factory);
    delegationMenuCmpRef.instance.delegatableThing = delegatableThing;
    // Trigger the delegation menu, causing it to open
    // Needs to be done asynchronously to wait for the component's view to initialize
    setTimeout((/**
     * @return {?}
     */
    () => delegationMenuCmpRef.instance.trigger.openMenu()), 0);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A service for keeping track of when items from various collections were last engaged with
class CyberUiTimeSinceLastEngagementService {
    /**
     * @param {?} firestoreBackend
     */
    constructor(firestoreBackend) {
        this.firestoreBackend = firestoreBackend;
        // A local representation of the last engagements with items
        // The top-level mapping is from collection ids (groupings)
        // to the lower-level mapping from item ids to engagement records
        this.state = new Map();
        // The top-level settings document where all of the persisted data lives
        this.collection = this.firestoreBackend.firestore
            .collection('settings')
            .doc('cyberUiTimeSinceLastEngagementService')
            .collection('engagementRecords');
        this.collection.stateChanges().subscribe((/**
         * @param {?} actions
         * @return {?}
         */
        actions => {
            actions.map((/**
             * @param {?} action
             * @return {?}
             */
            action => {
                /** @type {?} */
                const collectionId = action.payload.doc.id;
                // If this mapping changed, updated it locally
                if (action.type === 'added' || action.type === 'modified') {
                    /** @type {?} */
                    const mapping = action.payload.doc.data();
                    this.state.set(collectionId, mapping);
                }
                else {
                    // The mapping was deleted
                    this.state.delete(collectionId);
                }
            }));
        }));
    }
    // Returns the engagement record for the requested item from the given collection
    /**
     * @private
     * @param {?} collectionId
     * @param {?} itemId
     * @return {?}
     */
    getEngagementRecord(collectionId, itemId) {
        /** @type {?} */
        const mapping = this.state.get(collectionId);
        return mapping && mapping[itemId];
    }
    /**
     * @param {?} options
     * @return {?}
     */
    getTimeSinceLastEngagement(options) {
        /** @type {?} */
        const record = this.getEngagementRecord(options.collectionId, options.itemId);
        /** @type {?} */
        const timestamp = record && record.timestamp;
        /** @type {?} */
        let duration = undefined;
        if (timestamp !== undefined) {
            duration = Date.now() - timestamp;
        }
        return {
            timestamp: timestamp,
            duration: duration,
        };
    }
    // Updates the engagement record for the given item
    /**
     * @param {?} collectionId
     * @param {?} itemId
     * @return {?}
     */
    indicateEngagement(collectionId, itemId) {
        /** @type {?} */
        let engagementRecord = this.getEngagementRecord(collectionId, itemId);
        if (!engagementRecord) {
            engagementRecord = {
                timestamp: undefined
            };
        }
        engagementRecord.timestamp = Date.now();
        /** @type {?} */
        const docToUpdate = this.collection.doc(collectionId);
        /** @type {?} */
        const update = {
            [itemId]: engagementRecord
        };
        return docToUpdate.update(update).catch((/**
         * @param {?} error
         * @return {?}
         */
        error => {
            // One possible error is that the document doesn't exist yet
            if (error.code === 'not-found') {
                // Go ahead and create the document first
                return docToUpdate.set(update);
            }
            else {
                console.error(error);
            }
        }));
    }
}
CyberUiTimeSinceLastEngagementService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
CyberUiTimeSinceLastEngagementService.ctorParameters = () => [
    { type: CyberUiFirestoreBackend }
];
/** @nocollapse */ CyberUiTimeSinceLastEngagementService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiTimeSinceLastEngagementService_Factory() { return new CyberUiTimeSinceLastEngagementService(ɵɵinject(CyberUiFirestoreBackend)); }, token: CyberUiTimeSinceLastEngagementService, providedIn: "root" });
if (false) {
    /**
     * @type {?}
     * @private
     */
    CyberUiTimeSinceLastEngagementService.prototype.state;
    /**
     * @type {?}
     * @private
     */
    CyberUiTimeSinceLastEngagementService.prototype.collection;
    /** @type {?} */
    CyberUiTimeSinceLastEngagementService.prototype.firestoreBackend;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiTimeSinceLastEngagementModule {
}
CyberUiTimeSinceLastEngagementModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    CyberUiTimeSinceLastEngagementService
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Directive for automatically focusing on an element
// By default, the native 'autofocus' attribute on input elements only causes focus when the element first enters the DOM
// Since Angular sometimes keeps elements in the DOM when navigating to another route, the native autofocus attribute doesn't always work
// This directive provides more comprehensive support for autofocusing, including after navigation events
class CyberUiAutofocusDirective {
    /**
     * @param {?} el
     */
    constructor(el) {
        this.el = el;
        // If used without data binding, the value of `autofocus` will be the empty string
        // We'll assume the user wants autofocus behavior in that case
        // The only time we don't want autofocus is when `autofocous` is set to false via the input binding
        this.cyberUiAutofocus = true;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.cyberUiAutofocus === true || this.cyberUiAutofocus === '') {
            // Call focus() asynchronously to avoid 'Expression was changed after it was checked' errors
            setTimeout((/**
             * @return {?}
             */
            () => this.el.nativeElement.focus()), 0);
        }
    }
}
CyberUiAutofocusDirective.decorators = [
    { type: Directive, args: [{
                selector: '[cyberUiAutofocus]',
            },] }
];
/** @nocollapse */
CyberUiAutofocusDirective.ctorParameters = () => [
    { type: ElementRef }
];
CyberUiAutofocusDirective.propDecorators = {
    cyberUiAutofocus: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiAutofocusDirective.prototype.cyberUiAutofocus;
    /**
     * @type {?}
     * @private
     */
    CyberUiAutofocusDirective.prototype.el;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiAutofocusModule {
}
CyberUiAutofocusModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    CyberUiAutofocusDirective,
                ],
                exports: [
                    CyberUiAutofocusDirective,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// The base form field class
// It's expected that consumers will use subclasses of this class (e.g. TextField, BooleanField)
// rather than using this class directly
// For a list of subclasses, see the `fields` subdirectory
/**
 * @template MODEL_T, OPTIONS_T, CONFIG_T
 */
class FormField {
    /**
     * @param {?} options
     * @param {?} configConstructorCls
     */
    constructor(options, configConstructorCls) {
        this.config = new configConstructorCls(options);
    }
    // Formats a field value as a string
    // Can be overridden by subclasses of FormField
    /**
     * @param {?} value
     * @return {?}
     */
    format(value) {
        if (value === undefined) {
            return 'None';
        }
        else {
            return value.toString();
        }
    }
    // Utility function to get a model property
    /**
     * @param {?} model
     * @param {?} propertyName
     * @return {?}
     */
    getModelProperty(model, propertyName) {
        if ('getProperty' in model) {
            // If the model implements getProperty(), use that
            return ((/** @type {?} */ (model))).getProperty(propertyName);
        }
        else {
            // Otherwise, assume this is a simple literal object model
            return model[propertyName];
        }
    }
    /**
     * @param {?} model
     * @param {?} propertyName
     * @param {?} value
     * @return {?}
     */
    setModelProperty(model, propertyName, value) {
        if ('setProperty' in model) {
            // If the model implements setProperty(), use that
            return ((/** @type {?} */ (model))).setProperty(propertyName, value);
        }
        else {
            // Otherwise, assume this is a simple literal object model
            model[propertyName] = value;
        }
    }
}
if (false) {
    /** @type {?} */
    FormField.prototype.config;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const FormFieldElement = {
    NONE: 0,
    INPUT: 1,
    MAT_CHECKBOX: 2,
    MAT_SLIDE_TOGGLE: 3,
    MAT_SLIDER: 4,
    SELECT: 5,
    TEXTAREA: 6,
    DISCRETE_PROBABILITY_DISTRIBUTION: 7,
    CYBER_UI_FORM_FIELDS_LIST: 8,
};
FormFieldElement[FormFieldElement.NONE] = 'NONE';
FormFieldElement[FormFieldElement.INPUT] = 'INPUT';
FormFieldElement[FormFieldElement.MAT_CHECKBOX] = 'MAT_CHECKBOX';
FormFieldElement[FormFieldElement.MAT_SLIDE_TOGGLE] = 'MAT_SLIDE_TOGGLE';
FormFieldElement[FormFieldElement.MAT_SLIDER] = 'MAT_SLIDER';
FormFieldElement[FormFieldElement.SELECT] = 'SELECT';
FormFieldElement[FormFieldElement.TEXTAREA] = 'TEXTAREA';
FormFieldElement[FormFieldElement.DISCRETE_PROBABILITY_DISTRIBUTION] = 'DISCRETE_PROBABILITY_DISTRIBUTION';
FormFieldElement[FormFieldElement.CYBER_UI_FORM_FIELDS_LIST] = 'CYBER_UI_FORM_FIELDS_LIST';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template MODEL_T
 */
class CyberUiFormFieldsComponent {
    /**
     * @param {?} changeDetectorRef
     * @param {?} dialog
     */
    constructor(changeDetectorRef, dialog) {
        this.changeDetectorRef = changeDetectorRef;
        this.dialog = dialog;
        // Stream of change actions. If the any fields' model has changed, an event will be emitted.
        this.change = new EventEmitter();
        // Stream of save actions. If the model should be saved, an event will be emitted.
        this.save = new EventEmitter();
        // Expose the FormFieldElement enum to the template for use in comparisons
        this.elements = FormFieldElement;
    }
    /**
     * @return {?}
     */
    emitSaveAndReturnFalse() {
        this.save.emit();
        return false;
    }
    /**
     * @return {?}
     */
    getFields() {
        return (this.fields || (this.field && [this.field]) || ((/** @type {?} */ (this.model))).fields || ((/** @type {?} */ (this.model.constructor))).fields);
    }
    // Utility function to get a model property
    // TODO Remove redundancy with implementation on FormField class
    /**
     * @param {?} model
     * @param {?} propertyName
     * @return {?}
     */
    getModelProperty(model, propertyName) {
        if ('getProperty' in model) {
            // If the model implements getProperty(), use that
            return ((/** @type {?} */ (model))).getProperty(propertyName);
        }
        else {
            // Otherwise, assume this is a simple literal object model
            return model[propertyName];
        }
    }
    // TODO Remove redundancy with implementation on FormField class
    /**
     * @param {?} model
     * @param {?} propertyName
     * @param {?} value
     * @return {?}
     */
    setModelProperty(model, propertyName, value) {
        if ('setProperty' in model) {
            // If the model implements setProperty(), use that
            return ((/** @type {?} */ (model))).setProperty(propertyName, value);
        }
        else {
            // Otherwise, assume this is a simple literal object model
            model[propertyName] = value;
        }
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if ((changes.model && this.getFields() !== undefined) || (changes.fields !== undefined && changes.fields.currentValue !== undefined)) {
            // Initialize undefined model fields where necessary
            (this.getFields() || changes.fields.currentValue).forEach((/**
             * @param {?} field
             * @return {?}
             */
            field => {
                // Initialize discrete probabiliy distributions
                if (field.config.element === FormFieldElement.DISCRETE_PROBABILITY_DISTRIBUTION) {
                    if (this.getModelProperty(this.model, field.config.propertyName) === undefined) {
                        this.setModelProperty(this.model, field.config.propertyName, ((/** @type {?} */ (field.config))).outcomePresets.map((/**
                         * @param {?} outcome
                         * @return {?}
                         */
                        outcome => ({ outcome: outcome, probability: { value: '' } }))));
                    }
                }
            }));
        }
    }
    /**
     * @param {?} model
     * @param {?} propertyName
     * @param {?} newValue
     * @return {?}
     */
    onNgModelChange(model, propertyName, newValue) {
        this.setModelProperty(model, propertyName, newValue);
        this.change.emit();
    }
    /**
     * @param {?} cmp
     * @return {?}
     */
    openHelpDialog(cmp) {
        this.dialog.open(cmp);
    }
}
CyberUiFormFieldsComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-form-fields',
                template: "<!-- Fields may either be passed in via the 'fields' input, or defined on the model instance, or defined statically on the model's constructor -->\n<ng-container *ngFor=\"let field of getFields()\">\n  <mat-form-field *ngIf=\"field.config.useMatFormField\">\n    <!-- TextField template -->\n    <textarea\n      *ngIf=\"field.config.element === elements.TEXTAREA\"\n      matInput\n      matTextareaAutosize\n      [placeholder]=\"field.config.label\"\n      [required]=\"field.config.required\"\n      [cyberUiAutofocus]=\"field.config.autofocus\"\n      [ngModel]=\"field.getNgModelBoundValue(model)\"\n      (ngModelChange)=\"field.boundValueChangeHandler(model, $event)\"\n      (ngModelChange)=\"change.emit()\"\n      (keydown.enter)=\"field.config.saveOnEnter ? emitSaveAndReturnFalse() : null\"></textarea>\n    <!-- ChoiceField template -->\n    <mat-select\n        *ngIf=\"field.config.element === elements.SELECT\"\n        [placeholder]=\"field.config.label\"\n        [required]=\"field.config.required\"\n        [name]=\"field.config.propertyName\"\n        [ngModel]=\"getModelProperty(model, field.config.propertyName)\"\n        (ngModelChange)=\"onNgModelChange(model, field.config.propertyName, $event)\"\n        [cyberUiAutofocus]=\"field.autofocus\"\n        [multiple]=\"field.config.multiple\">\n      <mat-option *ngFor=\"let option of field.config.options\" [value]=\"option.value\">\n       {{option.label}}\n      </mat-option>\n    </mat-select>\n  </mat-form-field>\n  <!-- Help dialog placement when mat-form-field is used. Needs better styling / layout -->\n  <button type=\"button\" *ngIf=\"field.config.useMatFormField && field.config.helpDialog\" mat-icon-button (click)=\"openHelpDialog(field.config.helpDialog)\"><mat-icon>help</mat-icon></button>\n  <!-- BooleanField template -->\n  <div class=\"checkbox-container\" *ngIf=\"field.config.element === elements.MAT_CHECKBOX\">\n    <mat-checkbox\n        color=\"primary\"\n        [required]=\"field.config.required\"\n        [name]=\"field.config.propertyName\"\n        [ngModel]=\"getModelProperty(model, field.config.propertyName)\"\n        (ngModelChange)=\"onNgModelChange(model, field.config.propertyName, $event)\"\n        [cyberUiAutofocus]=\"field.config.autofocus\">\n      {{field.config.label}}\n    </mat-checkbox>\n    <button type=\"button\" *ngIf=\"field.config.helpDialog\" mat-icon-button (click)=\"openHelpDialog(field.config.helpDialog)\"><mat-icon>help</mat-icon></button>\n  </div>\n  <div class=\"slide-toggle-container\" *ngIf=\"field.config.element === elements.MAT_SLIDE_TOGGLE\">\n    <span>{{field.config.label}}&nbsp;</span>\n    <mat-slide-toggle\n        color=\"primary\"\n        [required]=\"field.config.required\"\n        [name]=\"field.config.propertyName\"\n        [ngModel]=\"getModelProperty(model, field.config.propertyName)\"\n        (ngModelChange)=\"onNgModelChange(model, field.config.propertyName, $event)\"\n        [cyberUiAutofocus]=\"field.config.autofocus\">\n    </mat-slide-toggle>\n    <button type=\"button\" *ngIf=\"field.config.helpDialog\" mat-icon-button (click)=\"openHelpDialog(field.config.helpDialog)\"><mat-icon>help</mat-icon></button>\n  </div>\n  <!-- ValueInNumericRangeField template -->\n  <div class=\"slider-container\" *ngIf=\"field.config.element === elements.MAT_SLIDER\">\n    <span>{{field.config.label}}&nbsp;</span>\n    <mat-slider\n        color=\"primary\"\n        [required]=\"field.config.required\"\n        [name]=\"field.config.propertyName\"\n        [ngModel]=\"getModelProperty(model, field.config.propertyName)\"\n        (ngModelChange)=\"onNgModelChange(model, field.config.propertyName, $event)\"\n        [max]=\"field.config.maxValue\"\n        [min]=\"field.config.minValue\"\n        [step]=\"field.config.step\"\n        [cyberUiAutofocus]=\"field.autofocus\">\n    </mat-slider>\n    <span>{{getModelProperty(model, field.config.propertyName)|number:field.config.digitinfo}}</span>\n    <button\n        mat-icon-button\n        *ngIf=\"getModelProperty(model, field.config.propertyName) !== undefined\"\n        (click)=\"setModelProperty(model, field.config.propertyName, undefined)\">\n      <mat-icon>close</mat-icon>\n    </button>\n    <!-- button needs type=\"button\" to avoid being interpreted as a submit button: https://stackoverflow.com/a/47186021/860525 -->\n    <button type=\"button\" *ngIf=\"field.config.helpDialog\" mat-icon-button (click)=\"openHelpDialog(field.config.helpDialog)\"><mat-icon>help</mat-icon></button>\n  </div>\n  <!-- DiscreteProbabilityDistributionField template -->\n  <!-- TODO Move this to a cyber-ui-discrete-probability-distribution-field component -->\n  <div\n      *ngIf=\"field.config.element === elements.DISCRETE_PROBABILITY_DISTRIBUTION\"\n      class=\"discrete-probability-distribution-field\">\n    <p>{{field.config.label}}</p>\n    <div class=\"outcome-probability-assignment\"\n         *ngFor=\"let assignment of getModelProperty(model, field.config.propertyName)\">\n      <mat-form-field [class.outcome]=\"true\">\n        <textarea\n            matInput\n            matTextareaAutosize\n            placeholder=\"Outcome\"\n            [(ngModel)]=\"assignment.outcome.label\"></textarea>\n      </mat-form-field>\n      <mat-form-field [class.probability]=\"true\">\n        <textarea\n            matInput\n            matTextareaAutosize\n            placeholder=\"Probability\"\n            [(ngModel)]=\"assignment.probability.value\"></textarea>\n      </mat-form-field>\n    </div>\n    <button\n        type=\"button\"\n        color=\"primary\"\n        mat-stroked-button\n        (click)=\"(getModelProperty(model, field.config.propertyName) === undefined && setModelProperty(model, field.config.propertyName, []) && false) || getModelProperty(model, field.config.propertyName).push({outcome: {label: ''}, probability: {value: ''}})\">ADD ANOTHER OUTCOME</button>\n  </div>\n  <!-- FieldListListField template -->\n  <!-- TODO Consider moving this to its own component at some point -->\n  <div\n      *ngIf=\"field.config.element === elements.CYBER_UI_FORM_FIELDS_LIST\"\n      class=\"field-list-list-field\">\n    <p>{{field.config.label}}</p>\n    <div *ngIf=\"getModelProperty(model, field.config.propertyName) !== undefined\">\n      <div\n          *ngFor=\"let item of getModelProperty(model, field.config.propertyName); let i = index\"\n          class=\"field-list-list-field-item\">\n        <!-- button needs type=\"button\" to avoid being interpreted as a submit button: https://stackoverflow.com/a/47186021/860525 -->\n        <button\n            type=\"button\"\n            mat-icon-button\n            color=\"primary\"\n            (click)=\"getModelProperty(model, field.config.propertyName).splice(i, 1)\">\n          <mat-icon aria-label=\"Remove item\">close</mat-icon>\n        </button>\n        <cyber-ui-form-fields\n            [fields]=\"field.config.fields\"\n            [model]=\"item\"\n            (change)=\"change.emit()\"\n            (save)=\"save.emit()\">\n        </cyber-ui-form-fields>\n      </div>\n    </div>\n    <button\n        type=\"button\"\n        color=\"primary\"\n        mat-stroked-button\n        (click)=\"(getModelProperty(model, field.config.propertyName) === undefined && setModelProperty(model, field.config.propertyName, []) && false) || getModelProperty(model, field.config.propertyName).push({}) && changeDetectorRef.detectChanges()\">ADD {{field.config.contentTypeLabel}}</button>\n  </div>\n</ng-container>\n",
                styles: [":host{display:flex;flex-direction:column}.slider-container{display:flex;align-items:center}.slider-container mat-slider{flex-grow:1}.outcome-probability-assignment{display:flex}.outcome-probability-assignment mat-form-field.outcome{flex-basis:320px;flex-grow:5;margin-right:16px}.outcome-probability-assignment mat-form-field.probability{flex-basis:80px;flex-grow:1}.field-list-list-field-item{display:flex;flex-direction:row;margin-bottom:18px}.field-list-list-field-item cyber-ui-form-fields{flex-grow:1}::ng-deep .mat-select-arrow,::ng-deep .mat-select-value{color:var(--text-color)}::ng-deep .mat-form-field-appearance-legacy .mat-form-field-label{color:var(--mat-list-subheader-color)}::ng-deep .mat-form-field-appearance-legacy .mat-form-field-underline{background-color:var(--low-opacity-color)}:host-context(.cdk-overlay-container) ::ng-deep .mat-select-arrow,:host-context(.cdk-overlay-container) ::ng-deep .mat-select-value{color:var(--light-background-text-color)}:host-context(.cdk-overlay-container) ::ng-deep .mat-form-field-appearance-legacy .mat-form-field-label{color:var(--light-background-subheader-color)}:host-context(.cdk-overlay-container) ::ng-deep .mat-form-field-appearance-legacy .mat-form-field-underline{background-color:var(--light-background-low-opacity-color)}"]
            }] }
];
/** @nocollapse */
CyberUiFormFieldsComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: MatDialog }
];
CyberUiFormFieldsComponent.propDecorators = {
    model: [{ type: Input }],
    fields: [{ type: Input }],
    field: [{ type: Input }],
    change: [{ type: Output }],
    save: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    CyberUiFormFieldsComponent.prototype.model;
    /** @type {?} */
    CyberUiFormFieldsComponent.prototype.fields;
    /** @type {?} */
    CyberUiFormFieldsComponent.prototype.field;
    /** @type {?} */
    CyberUiFormFieldsComponent.prototype.change;
    /** @type {?} */
    CyberUiFormFieldsComponent.prototype.save;
    /** @type {?} */
    CyberUiFormFieldsComponent.prototype.elements;
    /** @type {?} */
    CyberUiFormFieldsComponent.prototype.changeDetectorRef;
    /** @type {?} */
    CyberUiFormFieldsComponent.prototype.dialog;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiFormFieldsModule {
}
CyberUiFormFieldsModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule,
                    MatButtonModule,
                    MatCheckboxModule,
                    MatDialogModule,
                    MatIconModule,
                    MatInputModule,
                    MatSelectModule,
                    MatSliderModule,
                    MatSlideToggleModule,
                    CyberUiAutofocusModule,
                ],
                declarations: [
                    CyberUiFormFieldsComponent
                ],
                exports: [
                    CyberUiFormFieldsComponent
                ],
                entryComponents: [
                    CyberUiFormFieldsComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function FormFieldOptions() { }
if (false) {
    /** @type {?} */
    FormFieldOptions.prototype.label;
    /** @type {?} */
    FormFieldOptions.prototype.propertyName;
    /** @type {?|undefined} */
    FormFieldOptions.prototype.required;
    /** @type {?|undefined} */
    FormFieldOptions.prototype.autofocus;
    /** @type {?|undefined} */
    FormFieldOptions.prototype.helpText;
    /** @type {?|undefined} */
    FormFieldOptions.prototype.helpDialog;
    /** @type {?|undefined} */
    FormFieldOptions.prototype.element;
}
class FormFieldConfig {
    /**
     * @param {?} options
     * @param {?} supportedElements
     * @param {?} defaultElement
     */
    constructor(options, supportedElements, defaultElement) {
        this.label = options.label;
        this.propertyName = options.propertyName;
        this.element = this.getElement(options.element, supportedElements, defaultElement);
        this.required = this.getRequired(options.required);
        this.autofocus = this.getAutofocus(options.autofocus);
        this.helpText = options.helpText;
        this.helpDialog = options.helpDialog;
        this.useMatFormField = this.getUseMatFormField();
    }
    /**
     * @protected
     * @param {?} preference
     * @param {?} supported
     * @param {?} defaultElement
     * @return {?}
     */
    getElement(preference, supported, defaultElement) {
        if (preference !== undefined) {
            if (supported.indexOf(preference) === -1) {
                console.error(`Requested element`, FormFieldElement[preference], `is not supported by this field type; supported elements are:`, supported.map((/**
                 * @param {?} el
                 * @return {?}
                 */
                el => FormFieldElement[el])), `; proceeding with default element`, FormFieldElement[defaultElement]);
                return defaultElement;
            }
            else {
                return preference;
            }
        }
        // No preference was given, so use the default element for this field type
        return defaultElement;
    }
    /**
     * @private
     * @param {?} required
     * @return {?}
     */
    getRequired(required) {
        if (required !== undefined) {
            return required;
        }
        else {
            return false;
        }
    }
    /**
     * @private
     * @param {?} autofocus
     * @return {?}
     */
    getAutofocus(autofocus) {
        if (autofocus !== undefined) {
            return autofocus;
        }
        else {
            return false;
        }
    }
    // Logic to determine whether to use a mat-form-field component in the template or not
    // Since the config is immutable, the returned value can be cached on the config
    /**
     * @private
     * @return {?}
     */
    getUseMatFormField() {
        // TODO Return true only if input el or select el or textarea el
        // TODO Test this method is doing the right thing
        return (this.element === FormFieldElement.TEXTAREA ||
            this.element === FormFieldElement.SELECT);
    }
}
if (false) {
    /** @type {?} */
    FormFieldConfig.prototype.label;
    /** @type {?} */
    FormFieldConfig.prototype.propertyName;
    /** @type {?} */
    FormFieldConfig.prototype.required;
    /** @type {?} */
    FormFieldConfig.prototype.autofocus;
    /** @type {?} */
    FormFieldConfig.prototype.helpText;
    /** @type {?} */
    FormFieldConfig.prototype.helpDialog;
    /** @type {?} */
    FormFieldConfig.prototype.useMatFormField;
    /** @type {?} */
    FormFieldConfig.prototype.element;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BooleanFieldConfig extends FormFieldConfig {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, 
        // This field type supports MAT_CHECKBOX and MAT_SLIDE_TOGGLE elements
        [
            FormFieldElement.MAT_CHECKBOX,
            FormFieldElement.MAT_SLIDE_TOGGLE,
        ], 
        // The default UI element is MAT_CHECKBOX
        FormFieldElement.MAT_CHECKBOX);
    }
}
// This field type is for boolean values
// It also supports an initial `undefined` state
// Fields of this type are stored on the data model as true, false, or undefined
/**
 * @template MODEL_T
 */
class BooleanField extends FormField {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, BooleanFieldConfig);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Class for representing an option in the list of options for ChoiceField
/**
 * @template VALUE_T
 */
class Option {
    /**
     * @param {?} label
     * @param {?} value
     */
    constructor(label, value) {
        this.label = label;
        this.value = value;
    }
}
if (false) {
    /** @type {?} */
    Option.prototype.label;
    /** @type {?} */
    Option.prototype.value;
}
/**
 * @record
 * @template OPTION_VALUE_T
 */
function ChoiceFieldOptions() { }
if (false) {
    /** @type {?} */
    ChoiceFieldOptions.prototype.options;
    /** @type {?|undefined} */
    ChoiceFieldOptions.prototype.multiple;
}
/**
 * @template OPTION_VALUE_T
 */
class ChoiceFieldConfig extends FormFieldConfig {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, 
        // Supported elements for this field type are SELECT
        [FormFieldElement.SELECT], 
        // The default UI element for this field type is SELECT
        FormFieldElement.SELECT);
        this.options = options.options;
        this.multiple = this.getMultiple(options.multiple);
    }
    /**
     * @param {?} multiple
     * @return {?}
     */
    getMultiple(multiple) {
        if (multiple !== undefined) {
            return multiple;
        }
        else {
            return false;
        }
    }
}
if (false) {
    /** @type {?} */
    ChoiceFieldConfig.prototype.options;
    /** @type {?} */
    ChoiceFieldConfig.prototype.multiple;
}
// This field type for representing a choice from a discrete set of options
// The model's value will be whatever the underying selected option's value is
/**
 * @template MODEL_T, OPTION_VALUE_T
 */
class ChoiceField extends FormField {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, ChoiceFieldConfig);
    }
    // When displaying the value as a string, use the option's corresponding label
    /**
     * @param {?} value
     * @return {?}
     */
    format(value) {
        if (value === undefined) {
            return 'None';
        }
        else {
            /** @type {?} */
            const correspondingOption = this.config.options.find((/**
             * @param {?} option
             * @return {?}
             */
            option => option.value === value));
            if (correspondingOption) {
                return correspondingOption.label;
            }
            else {
                // TODO Log an error in some uniform way here
                return '[ERROR - COULD NOT RESOLVE VALUE]';
            }
        }
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function TextFieldOptions() { }
if (false) {
    /** @type {?|undefined} */
    TextFieldOptions.prototype.saveOnEnter;
}
class TextFieldConfig extends FormFieldConfig {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, 
        // Supported elements for this field type are TEXTAREA
        [FormFieldElement.TEXTAREA], 
        // The default UI element for this field type is TEXTAREA
        FormFieldElement.TEXTAREA);
        this.saveOnEnter = this.getSaveOnEnter(options.saveOnEnter);
    }
    /**
     * @param {?} saveOnEnter
     * @return {?}
     */
    getSaveOnEnter(saveOnEnter) {
        if (saveOnEnter !== undefined) {
            return saveOnEnter;
        }
        else {
            return false;
        }
    }
}
if (false) {
    /** @type {?} */
    TextFieldConfig.prototype.saveOnEnter;
}
// This field type is for various text such as labels or notes
// Other fields may subclass this field and provide alternate representations on the data model
// e.g. CommaSeparatedListField subclasses TextField and stores itself as a string[] on the model
/**
 * @template MODEL_T
 */
class TextField extends FormField {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, TextFieldConfig);
    }
    // Returns the string value to bind to ngModel in the template
    // Can be overridden in subclasses to perform additional logic
    /**
     * @param {?} model
     * @return {?}
     */
    getNgModelBoundValue(model) {
        return this.getModelProperty(model, this.config.propertyName);
    }
    // ngModel change handler to save the UI value into the proper format on the data model
    // @param `newValue` is the new value, shown in the UI, which triggered the change
    // Can be overridden in sublasses to deserialize the new value into a custom representation
    /**
     * @param {?} model
     * @param {?} newValue
     * @return {?}
     */
    boundValueChangeHandler(model, newValue) {
        this.setModelProperty(model, this.config.propertyName, newValue);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Represents a list of string values
// Shown in the UI as a comma-separated, user-editable list
// Note: on the data model, this field is stored as a string[]
/**
 * @template MODEL_T
 */
class CommaSeparatedListField extends TextField {
    // Since this field is stored on the model as a string[], we need to provide a custom
    // getNgModelBoundValue implementation which serializes the field value to a string
    /**
     * @param {?} model
     * @return {?}
     */
    getNgModelBoundValue(model) {
        // If the bound value is undefined, serialize the result to an empty string
        // This avoid the error of calling .join(...) on undefined
        // TODO Write a test that this field initializes correctly when the property is initially undefined
        if (this.getModelProperty(model, this.config.propertyName) === undefined) {
            return '';
        }
        else {
            return ((/** @type {?} */ (this.getModelProperty(model, this.config.propertyName)))).join(', ');
        }
    }
    // ngModel change handler to save the corresponding string[] onto the data model
    // @param `newValue` is the new value, shown in the UI, which triggered the change
    /**
     * @param {?} model
     * @param {?} newValue
     * @return {?}
     */
    boundValueChangeHandler(model, newValue) {
        this.setModelProperty(model, this.config.propertyName, newValue.split(',')
            // trim left and right space from items
            .map((/**
         * @param {?} tag
         * @return {?}
         */
        tag => tag.trim()))
            // filter out item that are empty post-trim
            .filter((/**
         * @param {?} tag
         * @return {?}
         */
        tag => tag.length > 0)));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function DiscreteProbabilityDistributionFieldOptions() { }
if (false) {
    /** @type {?|undefined} */
    DiscreteProbabilityDistributionFieldOptions.prototype.outcomePresets;
}
class DiscreteProbabilityDistributionFieldConfig extends FormFieldConfig {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, 
        // Supported elements for this field type are DISCRETE_PROBABILITY_DISTRIBUTION
        [FormFieldElement.DISCRETE_PROBABILITY_DISTRIBUTION], 
        // The default UI element for this field type is DISCRETE_PROBABILITY_DISTRIBUTION
        FormFieldElement.DISCRETE_PROBABILITY_DISTRIBUTION);
        this.outcomePresets = this.getOutcomePresets(options.outcomePresets);
    }
    /**
     * @param {?} outcomePresets
     * @return {?}
     */
    getOutcomePresets(outcomePresets) {
        return outcomePresets || [];
    }
}
if (false) {
    /** @type {?} */
    DiscreteProbabilityDistributionFieldConfig.prototype.outcomePresets;
}
// This field serializes on the model as a OutcomeProbabilityAssignment[]
/**
 * @template MODEL_T
 */
class DiscreteProbabilityDistributionField extends FormField {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, DiscreteProbabilityDistributionFieldConfig);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function FieldListListFieldOptions() { }
if (false) {
    /** @type {?} */
    FieldListListFieldOptions.prototype.fields;
    /** @type {?|undefined} */
    FieldListListFieldOptions.prototype.contentTypeLabel;
}
class FieldListListFieldConfig extends FormFieldConfig {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, 
        // Supported elements for this field type are CYBER_UI_FORM_FIELDS_LIST
        [FormFieldElement.CYBER_UI_FORM_FIELDS_LIST], 
        // The default UI element for this field type is CYBER_UI_FORM_FIELDS_LIST
        FormFieldElement.CYBER_UI_FORM_FIELDS_LIST);
        this.fields = options.fields;
        this.contentTypeLabel = this.getContentTypeLabel(options.contentTypeLabel);
    }
    /**
     * @param {?=} label
     * @return {?}
     */
    getContentTypeLabel(label) {
        return label || 'CHILD';
    }
}
if (false) {
    /** @type {?} */
    FieldListListFieldConfig.prototype.fields;
    /** @type {?} */
    FieldListListFieldConfig.prototype.contentTypeLabel;
}
// This field type is for when you need a list of field lists
/**
 * @template MODEL_T
 */
class FieldListListField extends FormField {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, FieldListListFieldConfig);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function ValueInNumericRangeFieldOptions() { }
if (false) {
    /** @type {?} */
    ValueInNumericRangeFieldOptions.prototype.minValue;
    /** @type {?} */
    ValueInNumericRangeFieldOptions.prototype.maxValue;
    /** @type {?|undefined} */
    ValueInNumericRangeFieldOptions.prototype.step;
    /** @type {?|undefined} */
    ValueInNumericRangeFieldOptions.prototype.digitsinfo;
}
class ValueInNumericRangeFieldConfig extends FormFieldConfig {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, 
        // Currently, MAT_SLIDER is the only supported UI element for this field type
        [FormFieldElement.MAT_SLIDER], 
        // Therefore, it is also the default UI element for this field type
        FormFieldElement.MAT_SLIDER);
        this.minValue = options.minValue;
        this.maxValue = options.maxValue;
        this.step = options.step;
        this.digitsinfo = options.digitsinfo;
    }
}
if (false) {
    /** @type {?} */
    ValueInNumericRangeFieldConfig.prototype.minValue;
    /** @type {?} */
    ValueInNumericRangeFieldConfig.prototype.maxValue;
    /** @type {?} */
    ValueInNumericRangeFieldConfig.prototype.step;
    /** @type {?} */
    ValueInNumericRangeFieldConfig.prototype.digitsinfo;
}
/**
 * @template MODEL_T
 */
class ValueInNumericRangeField extends FormField {
    /**
     * @param {?} options
     */
    constructor(options) {
        super(options, ValueInNumericRangeFieldConfig);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// This field is a user-editable, Markdown-formatted text field that the user can use to track the completion criteria for a thing
// A typical example of the contents of this field would be a bulleted list of human-readable criteria
// Rationale for inclusion as a common field: task-like things commonly have some notion of completion criteria
// The property name is suffixed with 'Text' in case a more technical representation of the criteria are to be stored at 'completionCriteria'
/** @type {?} */
const COMPLETION_CRITERIA_TEXT_FIELD = new TextField({
    label: 'Completion criteria',
    propertyName: 'completionCriteriaText',
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A field representing the user's current energy level
/** @type {?} */
const ENERGY_LEVEL_FIELD = getEnergyLevelField(1, 5);
/**
 * @param {?} minValue
 * @param {?} maxValue
 * @return {?}
 */
function getEnergyLevelField(minValue, maxValue) {
    return new ValueInNumericRangeField({
        label: 'Energy level',
        propertyName: 'energyLevel',
        minValue: minValue,
        maxValue: maxValue,
    });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DEFAULT_EDIT_DIALOG_TITLE = 'Edit content';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// This needs to be defined here rather than on EditDialogService in order to avoid 
// a circular reference between the component and service
/**
 * @param {?} saveFn
 * @param {?} snackbar
 * @param {?} ctx
 * @return {?}
 */
function saveActionHandler(saveFn, snackbar, ctx) {
    // Close the dialog. If the save fails, refer the user to the console logs to recover their unsaved data
    ctx.dialogRef.close();
    saveFn().then((/**
     * @return {?}
     */
    () => snackbar.open('Saved')), (/**
     * @param {?} error
     * @return {?}
     */
    error => {
        console.error(error);
        snackbar.open('Failed to save - check console logs');
    }));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiEditDialogComponent {
    /**
     * @param {?} dialogRef
     * @param {?} params
     * @param {?} snackbar
     */
    constructor(dialogRef, params, snackbar) {
        this.dialogRef = dialogRef;
        this.params = params;
        this.snackbar = snackbar;
        // Update the dialog title to reflect the content type being edited
        this.title = this.getTitle(params.contentTypeLabel);
    }
    /**
     * @return {?}
     */
    saveAndClose() {
        // if there was a save function provided, or if the model exposes
        // a save method, call that and close the dialog
        // TODO Test that model.save() gets called if params.saveFn is not provided
        // TODO Test that model.save() gets called with the correct 'this' binding
        // TODO Remove 'any' types once params.model uses CyberUiSavableModel type
        /** @type {?} */
        let saveFn = this.params.saveFn;
        if (!saveFn && ((/** @type {?} */ (this.params.model))).save) {
            saveFn = (/**
             * @return {?}
             */
            () => ((/** @type {?} */ (this.params.model))).save());
        }
        if (saveFn) {
            saveActionHandler(saveFn, this.snackbar, {
                dialogRef: this.dialogRef
            });
        }
        else {
            // Close the containing dialog without taking any further action
            this.dialogRef.close();
        }
    }
    /**
     * @param {?} contentTypeLabel
     * @return {?}
     */
    getTitle(contentTypeLabel) {
        if (contentTypeLabel) {
            return `Edit ${contentTypeLabel}`;
        }
        else {
            return DEFAULT_EDIT_DIALOG_TITLE;
        }
    }
}
CyberUiEditDialogComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-edit-dialog',
                template: "<div mat-dialog-title>{{title}}</div>\n<mat-dialog-content>\n  <cyber-ui-form-fields\n      [model]=\"params.model\"\n      [fields]=\"params.fields\"\n      (save)=\"saveAndClose()\">\n  </cyber-ui-form-fields>\n</mat-dialog-content>\n<mat-dialog-actions>\n  <button\n      mat-button\n      *ngFor=\"let action of params.actions\"\n      cyberUiActionContext\n      #actionContext=\"cyberUiActionContext\"\n      (click)=\"action.handler(actionContext)\"\n    >{{action.label}}</button>\n</mat-dialog-actions>\n"
            }] }
];
/** @nocollapse */
CyberUiEditDialogComponent.ctorParameters = () => [
    { type: MatDialogRef },
    { type: undefined, decorators: [{ type: Inject, args: [MAT_DIALOG_DATA,] }] },
    { type: MatSnackBar }
];
if (false) {
    /** @type {?} */
    CyberUiEditDialogComponent.prototype.title;
    /** @type {?} */
    CyberUiEditDialogComponent.prototype.dialogRef;
    /** @type {?} */
    CyberUiEditDialogComponent.prototype.params;
    /** @type {?} */
    CyberUiEditDialogComponent.prototype.snackbar;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
const ɵ0 = /**
 * @param {?} ctx
 * @return {?}
 */
(ctx) => ctx.dialogRef.close();
/** @type {?} */
const CYBER_UI_SHOW_EDIT_DIALOG_CANCEL_ACTION = {
    label: 'CANCEL',
    handler: (ɵ0),
};
class CyberUiEditDialogService {
    /**
     * @param {?} dialog
     * @param {?} snackbar
     */
    constructor(dialog, snackbar) {
        this.dialog = dialog;
        this.snackbar = snackbar;
    }
    /**
     * @param {?} params
     * @return {?}
     */
    showEditDialog(params) {
        // Fill in empty params with reasonable defaults
        if (params.actions === undefined) {
            params.actions = [];
            // If the model has a 'save' method, add a 'save' action which calls it.
            if (typeof ((/** @type {?} */ (params.model))).save === 'function') {
                params.actions.push(this.getSaveAction((/**
                 * @return {?}
                 */
                () => ((/** @type {?} */ (params.model))).save())));
            }
            params.actions.push(CYBER_UI_SHOW_EDIT_DIALOG_CANCEL_ACTION);
        }
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            /** @type {?} */
            const dialogRef = this.dialog.open(CyberUiEditDialogComponent, { data: params });
            dialogRef.afterClosed().subscribe((/**
             * @return {?}
             */
            () => { }), (/**
             * @param {?} error
             * @return {?}
             */
            error => reject(error)), (/**
             * @return {?}
             */
            () => resolve(params.model)));
        }));
    }
    // Given a save function, returns a save Action for use with CyberUiEditDialogService
    /**
     * @param {?} saveFn
     * @return {?}
     */
    getSaveAction(saveFn) {
        return {
            label: 'SAVE',
            handler: (/**
             * @param {?} ctx
             * @return {?}
             */
            ctx => saveActionHandler(saveFn, this.snackbar, ctx)),
        };
    }
}
CyberUiEditDialogService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
CyberUiEditDialogService.ctorParameters = () => [
    { type: MatDialog },
    { type: MatSnackBar }
];
/** @nocollapse */ CyberUiEditDialogService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiEditDialogService_Factory() { return new CyberUiEditDialogService(ɵɵinject(MatDialog), ɵɵinject(MatSnackBar)); }, token: CyberUiEditDialogService, providedIn: "root" });
if (false) {
    /** @type {?} */
    CyberUiEditDialogService.prototype.dialog;
    /** @type {?} */
    CyberUiEditDialogService.prototype.snackbar;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiEditDialogModule {
}
CyberUiEditDialogModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    CyberUiEditDialogComponent,
                ],
                imports: [
                    CommonModule,
                    CyberUiActionContextModule,
                    CyberUiFormFieldsModule,
                    MatButtonModule,
                    MatDialogModule,
                    MatSnackBarModule,
                ],
                exports: [
                    CyberUiEditDialogComponent,
                ],
                providers: [
                    CyberUiEditDialogService,
                ],
                entryComponents: [
                    CyberUiEditDialogComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// The default timebox duration, in minutes
/** @type {?} */
const CYBER_UI_MINDFULLY_ATTEND_TO_TOPIC_DEFAULT_TIMEBOX_DURATION = 50;
/**
 * @template DATA_T
 */
class CyberUiTopicRegistration {
    /**
     * @param {?} options
     */
    constructor(options) {
        this.label = options.label;
        this.slug = this.getSlug(options);
        this.labelWhenUsedInASentence = this.getLabelWhenUsedInASentence(options);
        this.importance = this.getImportance(options);
        this.timeboxDuration = this.getTimeboxDuration(options);
        this.data = options.data;
    }
    /**
     * @param {?} options
     * @return {?}
     */
    getLabelWhenUsedInASentence(options) {
        return options.labelWhenUsedInASentence || this.label.toLocaleLowerCase();
    }
    /**
     * @param {?} options
     * @return {?}
     */
    getImportance(options) {
        /** @type {?} */
        const importance = options.importance;
        if (importance === undefined)
            console.error('Required property `importance` was undefined');
        if (importance < 0 || importance > 1)
            console.error('Required property `importance` was not in the domain [0, 1], which is required');
        return importance;
    }
    /**
     * @param {?} options
     * @return {?}
     */
    getSlug(options) {
        return options.slug || slugify(options.label, { lower: true });
    }
    /**
     * @param {?} options
     * @return {?}
     */
    getTimeboxDuration(options) {
        return options.timeboxDuration || CYBER_UI_MINDFULLY_ATTEND_TO_TOPIC_DEFAULT_TIMEBOX_DURATION;
    }
}
if (false) {
    /** @type {?} */
    CyberUiTopicRegistration.prototype.label;
    /** @type {?} */
    CyberUiTopicRegistration.prototype.slug;
    /** @type {?} */
    CyberUiTopicRegistration.prototype.labelWhenUsedInASentence;
    /** @type {?} */
    CyberUiTopicRegistration.prototype.importance;
    /** @type {?} */
    CyberUiTopicRegistration.prototype.timeboxDuration;
    /** @type {?} */
    CyberUiTopicRegistration.prototype.data;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A human readable label describing this provider
/** @type {?} */
const CYBER_UI_MINDFULLY_ATTEND_TO_TOPIC_TASK_PROVIDER_LABEL = 'Mindful attention to topics';
// A string identifier of this provider for use in e.g. settings lists
/** @type {?} */
const CYBER_UI_MINDFULLY_ATTEND_TO_TOPIC_TASK_PROVIDER_ID = 'CYBER_UI_MINDFULLY_ATTEND_TO_TOPIC_TASK_PROVIDER';
// The regexp used to stochastically generate the primary action label
/** @type {?} */
const PRIMARY_ACTION_LABEL_GENERATOR_REGEX = /(OK, LET'S GO|I'M READY|GET STARTED|START (IT|TIMEBOX|SESSION|HARD FOCUS SESSION))(!)?/;
// The regexp used to stochastically generate the secondary action label
/** @type {?} */
const SKIP_ACTION_LABEL_GENERATOR_REGEX = /NAH|NOT RIGHT NOW/;
// The regexp used to stochastically generate the task label
/** @type {?} */
const TASK_LABEL_GENERATOR_REGEX = /(Start|Spend) a( DURATION_TOKEN)? hard focus session( with the intention of)? mindfully attending to TOPIC_LABEL_TOKEN(, and log follow-ups as appropriate)?/;
// Provides tasks around mindfully attending to topics
// Example task statement:
// Start a 50-minute hard focus session with the intention to mindfully attend to systemization, and log follow-ups as appropriate
class CyberUiMindfullyAttendToTopicTaskProvider {
    constructor() {
        this.responses = new EventEmitter();
        this.topicRegistrations = [];
        // A copy of the list of topic registrations for external consumers
        this.topicRegistrationsSubject = new ReplaySubject(1);
        this.tasks = new ReplaySubject(1);
        this.primaryActionLabelRandExp = new RandExp(PRIMARY_ACTION_LABEL_GENERATOR_REGEX);
        this.skipActionLabelRandExp = new RandExp(SKIP_ACTION_LABEL_GENERATOR_REGEX);
        this.taskLabelRandExp = new RandExp(TASK_LABEL_GENERATOR_REGEX);
        // Initialize the tasks observable with an empty list
        // This way, the task provider won't wait indefinitely to receive a response
        // from this task provider, if no topics have been registered
        this.tasks.next([]);
        // Initialize the registrations observable with an empty list, to reflect the initial state
        this.topicRegistrationsSubject.next([]);
    }
    // Registers topics with the provider
    /**
     * @param {?} options
     * @return {?}
     */
    registerTopics(options) {
        if (options.clearExisting) {
            this.topicRegistrations = [];
        }
        for (let topicOptions of options.topics) {
            /** @type {?} */
            const topicRegistration = new CyberUiTopicRegistration(topicOptions);
            this.topicRegistrations.push(topicRegistration);
        }
        // Update the list of topic registrations that consumers might be listening to
        // Note: .slice() makes a shallow copy of the original array
        this.topicRegistrationsSubject.next(this.topicRegistrations.slice());
        // Update the list of tasks produced by this provider
        this.updateTasks();
        return;
    }
    // Utility method to get the current number of topic registrations
    /**
     * @return {?}
     */
    getTopicRegistrationsCount() {
        return this.topicRegistrations.length;
    }
    // Returns the topic registration matching the given slug
    /**
     * @param {?} slug
     * @return {?}
     */
    getTopicRegistrationFromSlug(slug) {
        return this.topicRegistrations.find((/**
         * @param {?} registration
         * @return {?}
         */
        registration => registration.slug === slug));
    }
    // Returns an observable of the current list of topic registrations
    /**
     * @return {?}
     */
    getTopicRegistrations() {
        return this.topicRegistrationsSubject;
    }
    /**
     * @return {?}
     */
    getTasks() {
        return this.tasks.pipe(map((/**
         * @param {?} tasks
         * @return {?}
         */
        tasks => {
            return {
                tasks: tasks
            };
        })));
    }
    // Updates the list of tasks produced by this provider, in response to a change in topic registrations
    /**
     * @private
     * @return {?}
     */
    updateTasks() {
        /** @type {?} */
        const tasks = [];
        this.topicRegistrations.forEach((/**
         * @param {?} topicRegistration
         * @return {?}
         */
        topicRegistration => {
            /** @type {?} */
            const label = this.getTaskLabelForTopic(topicRegistration);
            tasks.push({
                label: label,
                actions: this.getActions(label, topicRegistration),
                topic: topicRegistration,
            });
        }));
        this.tasks.next(tasks);
    }
    /**
     * @private
     * @param {?} taskLabel
     * @param {?} topicRegistration
     * @return {?}
     */
    getActions(taskLabel, topicRegistration) {
        /** @type {?} */
        const actions = [];
        /** @type {?} */
        const primaryAction = {
            label: this.getPrimaryActionLabel(),
            handler: (/**
             * @return {?}
             */
            () => {
                this.responses.next({
                    taskLabel: taskLabel,
                    actionLabel: primaryAction.label,
                    canonicalActionSlug: 'proceed',
                    topicRegistration: topicRegistration
                });
            })
        };
        actions.push(primaryAction);
        /** @type {?} */
        const skipAction = {
            label: this.getSkipActionLabel(),
            handler: (/**
             * @return {?}
             */
            () => {
                this.responses.next({
                    taskLabel: taskLabel,
                    actionLabel: skipAction.label,
                    canonicalActionSlug: 'skip',
                    topicRegistration: topicRegistration
                });
            })
        };
        actions.push(skipAction);
        return actions;
    }
    /**
     * @private
     * @param {?} topic
     * @return {?}
     */
    getTaskLabelForTopic(topic) {
        return this.taskLabelRandExp.gen()
            .replace('TOPIC_LABEL_TOKEN', topic.labelWhenUsedInASentence)
            .replace('DURATION_TOKEN', `${topic.timeboxDuration}-minute`);
    }
    /**
     * @private
     * @return {?}
     */
    getPrimaryActionLabel() {
        return this.primaryActionLabelRandExp.gen();
    }
    /**
     * @private
     * @return {?}
     */
    getSkipActionLabel() {
        return this.skipActionLabelRandExp.gen();
    }
}
// For convenience, store the task provider's suggested id as a static class property
CyberUiMindfullyAttendToTopicTaskProvider.id = CYBER_UI_MINDFULLY_ATTEND_TO_TOPIC_TASK_PROVIDER_ID;
CyberUiMindfullyAttendToTopicTaskProvider.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
CyberUiMindfullyAttendToTopicTaskProvider.ctorParameters = () => [];
/** @nocollapse */ CyberUiMindfullyAttendToTopicTaskProvider.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiMindfullyAttendToTopicTaskProvider_Factory() { return new CyberUiMindfullyAttendToTopicTaskProvider(); }, token: CyberUiMindfullyAttendToTopicTaskProvider, providedIn: "root" });
if (false) {
    /** @type {?} */
    CyberUiMindfullyAttendToTopicTaskProvider.id;
    /** @type {?} */
    CyberUiMindfullyAttendToTopicTaskProvider.prototype.responses;
    /**
     * @type {?}
     * @private
     */
    CyberUiMindfullyAttendToTopicTaskProvider.prototype.topicRegistrations;
    /**
     * @type {?}
     * @private
     */
    CyberUiMindfullyAttendToTopicTaskProvider.prototype.topicRegistrationsSubject;
    /**
     * @type {?}
     * @private
     */
    CyberUiMindfullyAttendToTopicTaskProvider.prototype.tasks;
    /**
     * @type {?}
     * @private
     */
    CyberUiMindfullyAttendToTopicTaskProvider.prototype.primaryActionLabelRandExp;
    /**
     * @type {?}
     * @private
     */
    CyberUiMindfullyAttendToTopicTaskProvider.prototype.skipActionLabelRandExp;
    /**
     * @type {?}
     * @private
     */
    CyberUiMindfullyAttendToTopicTaskProvider.prototype.taskLabelRandExp;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const TASK_SUGGESTION_SERVICE = new InjectionToken('TaskSuggestionService');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A highly experimental registry for collecting tasks from multiple providers
//
// I've found attempting to collect all tasks into a single canonical source to be less intuitive
// than one might initially expect. In practice, tasks naturally arise from multiple sources.
// For example, here are three common sources:
//
// 1) a one-for-one "list of tasks" that might come from a database or a simple text file
// 2) a list of parameterized, potentially recurring task templates instantiated in various ways
//    (these might not be one-for-one; a task template such as "Clean the {bedroom, bathroom, kitchen}"
//     could be instantiated into three separate tasks)
// 3) "TODO" annotations in project files
//
// By using a task provider registry, downstream consumers such as task suggestion services don't need to
// worry about where the tasks come from. They can just work with the tasks that they are given.
/**
 * @template TASK_T
 */
class TaskProviderRegistryService {
    constructor() {
        // The list of task providers registered in the registry
        this.providers = new Map();
    }
    // Registers the given task provider into the registry
    /**
     * @param {?} id
     * @param {?} provider
     * @return {?}
     */
    registerProvider(id, provider) {
        if (this.providers.has(id)) {
            console.error(`Refusing to register provider with duplicated id '${id}'`);
        }
        else {
            this.providers.set(id, provider);
        }
    }
    // Returns a copy of the list of current providers
    /**
     * @return {?}
     */
    getProviders() {
        return Array.from(this.providers.values());
    }
    // Returns an observable emitting the current list of tasks (live-updating after each task provider returns)
    // To get an observable emitting only the final complete list of tasks, call `.last()` on the returned observable
    /**
     * @param {?} options
     * @return {?}
     */
    getTasks(options) {
        // Maintain a mapping from providers to the latest tasks they've returned
        // Note that a given provider may emit multiple responses over time
        // In that case, we need to replace the old set of tasks with the new
        // As a result, just keeping a flat list of tasks returned so far is insufficient
        // We need to know which provider each task came from in order to do the replacement
        /** @type {?} */
        const tasksByProvider = new Map();
        /** @type {?} */
        const providerResponses = [];
        this.providers.forEach((/**
         * @param {?} provider
         * @param {?} providerId
         * @return {?}
         */
        (provider, providerId) => {
            // Only get the tasks from the providers indicated in the request (or all, if no providers were specified)
            if (options.taskProviders === undefined || options.taskProviders.indexOf(providerId) > -1) {
                providerResponses.push({
                    providerId: providerId,
                    responseObservable: provider.getTasks(options)
                });
            }
        }));
        /** @type {?} */
        let numProviderCompletions = 0;
        // If there are no active providers, emit an empty response
        if (providerResponses.length === 0) {
            return of({ suggestions: [] });
        }
        // If there are active providers, combine their responses into a single observable
        /** @type {?} */
        let observable = Observable.create((/**
         * @param {?} subscriber
         * @return {?}
         */
        (subscriber) => {
            merge(providerResponses
                .map((/**
             * @param {?} providerResponse
             * @return {?}
             */
            providerResponse => {
                /** @type {?} */
                let providerResponseObservable = providerResponse.responseObservable;
                // If the request has a deadline, terminate all provider response observables at the expiration of the deadline
                if (options.deadline) {
                    providerResponseObservable = providerResponseObservable.pipe(timeout(options.deadline));
                }
                providerResponseObservable.subscribe((/**
                 * @param {?} response
                 * @return {?}
                 */
                (response) => {
                    // Update the tasks associated with this provider
                    // Either they were previously undefined (in the case of the initial provider response),
                    // or they were previously defined but have become outdated due to the newer response,
                    // which might reflect updates to the provider's data store (new tasks, canceled tasks, etc.)
                    tasksByProvider.set(providerResponse.providerId, response.tasks);
                    /** @type {?} */
                    let result = [];
                    // Collect the full list of tasks from the map
                    // It would be nice to use flatMap here, but it's not supported by browsers yet
                    tasksByProvider.forEach((/**
                     * @param {?} tasks
                     * @param {?} providerId
                     * @return {?}
                     */
                    (tasks, providerId) => result = result.concat(tasks.map((/**
                     * @param {?} task
                     * @return {?}
                     */
                    task => {
                        return {
                            task: task,
                            providerId: providerId
                        };
                    })))));
                    subscriber.next({
                        suggestions: result
                    });
                }), (/**
                 * @param {?} error
                 * @return {?}
                 */
                error => {
                    console.error(error);
                    numProviderCompletions += 1;
                    // If this is the last provider to complete/error, mark the subscriber as complete
                    // (there won't be any more events after this)
                    if (numProviderCompletions === providerResponses.length) {
                        subscriber.complete();
                    }
                }), (/**
                 * @return {?}
                 */
                () => {
                    numProviderCompletions += 1;
                    // If this is the last provider to complete/error, mark the subscriber as complete
                    // (there won't be any more events after this)
                    if (numProviderCompletions === providerResponses.length) {
                        subscriber.complete();
                    }
                }));
            })));
        }));
        // If options.oneshot is set, return only the final response to the requester
        if (options.oneshot) {
            observable = observable.pipe(last());
        }
        return observable;
    }
}
TaskProviderRegistryService.decorators = [
    { type: Injectable }
];
if (false) {
    /**
     * @type {?}
     * @private
     */
    TaskProviderRegistryService.prototype.providers;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A component for showing a list of minimal task cards
/**
 * @template TASK_T
 */
class CyberUiMinimalTaskCardListComponent {
    // The tasks to display in the accordion
    /**
     * @param {?} tasks
     * @return {?}
     */
    set tasks(tasks) {
        if (this._tasks) {
            // Stop listening to the old tasks observable
            this.tasksSubscription.unsubscribe();
        }
        this._tasks = tasks;
    }
}
CyberUiMinimalTaskCardListComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-minimal-task-card-list',
                template: "<mat-spinner *ngIf=\"(_tasks | async) === null\"></mat-spinner>\n\n<ng-container *ngIf=\"(_tasks | async) !== null\">\n  <cyber-ui-minimal-task-card\n    *ngFor=\"let task of (_tasks | async)\"\n    [task]=\"task\">\n  </cyber-ui-minimal-task-card>\n</ng-container>",
                styles: ["cyber-ui-minimal-task-card:not(:last-child){margin-bottom:24px}:host mat-spinner{margin-top:64px}"]
            }] }
];
CyberUiMinimalTaskCardListComponent.propDecorators = {
    tasks: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiMinimalTaskCardListComponent.prototype._tasks;
    /** @type {?} */
    CyberUiMinimalTaskCardListComponent.prototype.tasksSubscription;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A relatively minimal display component for a task as a Material card
// TODO The logic in this component is almost identical to that of minimal-expansion-panel; figure out a way to reduce the redundancy
/**
 * @template TASK_T
 */
class CyberUiMinimalTaskCardComponent {
    /**
     * @param {?} resolver
     */
    constructor(resolver) {
        this.resolver = resolver;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.task) {
            // If the task has changed, update the body component in the view container
            this.updateBodyComponent();
            this.updateActionsDisplayComponent();
        }
    }
    // There are some times we need to do work even if the task reference hasn't changed
    // e.g. if certain task properties have been mutated, we need to do some updates here
    /**
     * @return {?}
     */
    ngDoCheck() {
        if (this.actionsDisplayComponent !== this.task.actionsDisplayComponent) {
            this.updateActionsDisplayComponent();
        }
        else if (this.bodyComponent !== this.task.bodyComponent) {
            this.updateBodyComponent();
        }
    }
    /**
     * @return {?}
     */
    updateActionsDisplayComponent() {
        // Clear the existing actions display component, if one exists
        if (this.actionsViewContainer) {
            this.actionsViewContainer.clear();
        }
        if (this.task.actionsDisplayComponent) {
            /** @type {?} */
            const factory = this.resolver.resolveComponentFactory(this.task.actionsDisplayComponent);
            /** @type {?} */
            const componentRef = this.actionsViewContainer.createComponent(factory);
            // Set the 'task' input on the component to the current task
            componentRef.instance.actions = this.task.actions;
        }
        this.actionsDisplayComponent = this.task.actionsDisplayComponent;
    }
    /**
     * @return {?}
     */
    updateBodyComponent() {
        // Clear the existing body component, if one exists
        this.bodyViewContainer.clear();
        // Insert a new body component, if one is given
        if (this.task.bodyComponent) {
            /** @type {?} */
            const factory = this.resolver.resolveComponentFactory(this.task.bodyComponent);
            /** @type {?} */
            const componentRef = this.bodyViewContainer.createComponent(factory);
            // Set the 'task' input on the component to the current task
            componentRef.instance.task = this.task;
        }
        // Save a reference to the current body component class
        this.bodyComponent = this.task.bodyComponent;
    }
}
CyberUiMinimalTaskCardComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-minimal-task-card',
                template: "<mat-card [class.cyber-ui-custom-actions-display-component]=\"task.actionsDisplayComponent\">\n  <mat-card-title>{{task.label}}</mat-card-title>\n  <ng-template #body></ng-template>\n  <ng-content></ng-content>\n  <div class=\"cyber-ui-actions-display-component-container\">\n    <ng-template #actions></ng-template>\n  </div>\n  <mat-card-actions *ngIf=\"!task.actionsDisplayComponent && task.actions !== undefined && task.actions.length > 0\">\n    <button\n      mat-button\n      cyberUiActionContext\n      #actionContext=\"cyberUiActionContext\"\n      *ngFor=\"let action of task.actions\"\n      (click)=\"action.handler(actionContext.getContext())\"\n      >{{action.label}}</button>\n  </mat-card-actions>\n</mat-card>",
                styles: [":host{display:block}.cyber-ui-actions-display-component-container:not(:empty){margin:0 -16px}.mat-card.cyber-ui-custom-actions-display-component{padding-bottom:0;overflow:hidden}::ng-deep mat-card-title.mat-card-title{line-height:normal}"]
            }] }
];
/** @nocollapse */
CyberUiMinimalTaskCardComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver }
];
CyberUiMinimalTaskCardComponent.propDecorators = {
    bodyViewContainer: [{ type: ViewChild, args: ['body', { read: ViewContainerRef, static: true },] }],
    actionsViewContainer: [{ type: ViewChild, args: ['actions', { read: ViewContainerRef, static: true },] }],
    task: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiMinimalTaskCardComponent.prototype.bodyViewContainer;
    /** @type {?} */
    CyberUiMinimalTaskCardComponent.prototype.actionsViewContainer;
    /** @type {?} */
    CyberUiMinimalTaskCardComponent.prototype.task;
    /** @type {?} */
    CyberUiMinimalTaskCardComponent.prototype.bodyComponent;
    /** @type {?} */
    CyberUiMinimalTaskCardComponent.prototype.actionsDisplayComponent;
    /**
     * @type {?}
     * @private
     */
    CyberUiMinimalTaskCardComponent.prototype.resolver;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiMinimalTaskCardModule {
}
CyberUiMinimalTaskCardModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatButtonModule,
                    MatCardModule,
                    CyberUiActionContextModule,
                ],
                declarations: [
                    CyberUiMinimalTaskCardComponent,
                ],
                exports: [
                    CyberUiMinimalTaskCardComponent,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiMinimalTaskCardListModule {
}
CyberUiMinimalTaskCardListModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    CyberUiMinimalTaskCardModule,
                    MatProgressSpinnerModule,
                ],
                declarations: [
                    CyberUiMinimalTaskCardListComponent,
                ],
                exports: [
                    CyberUiMinimalTaskCardListComponent,
                ],
                entryComponents: [
                    // This component will often be dynamically generated and provided to cyber-ui-work-on-tasks-workflow
                    CyberUiMinimalTaskCardListComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiTasksModule {
}
CyberUiTasksModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                declarations: [],
                exports: [],
                providers: [
                    TaskProviderRegistryService
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A relatively minimal display component for a task as a Material expansion panel
// TODO The logic in this component is almost identical to that of minimal-task-card; figure out a way to reduce the redundancy
/**
 * @template TASK_T
 */
class CyberUiMinimalTaskExpansionPanelComponent {
    /**
     * @param {?} resolver
     */
    constructor(resolver) {
        this.resolver = resolver;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.task) {
            // If the task has changed, update the body component in the view container
            this.updateBodyComponent();
            this.updateActionsDisplayComponent();
        }
    }
    // There are some times we need to do work even if the task reference hasn't changed
    // e.g. if certain task properties have been mutated, we need to do some updates here
    /**
     * @return {?}
     */
    ngDoCheck() {
        if (this.actionsDisplayComponent !== this.task.actionsDisplayComponent) {
            this.updateActionsDisplayComponent();
        }
        else if (this.bodyComponent !== this.task.bodyComponent) {
            this.updateBodyComponent();
        }
    }
    /**
     * @return {?}
     */
    updateActionsDisplayComponent() {
        // Clear the existing actions display component, if one exists
        if (this.actionsViewContainer) {
            this.actionsViewContainer.clear();
        }
        if (this.task.actionsDisplayComponent) {
            /** @type {?} */
            const factory = this.resolver.resolveComponentFactory(this.task.actionsDisplayComponent);
            /** @type {?} */
            const componentRef = this.actionsViewContainer.createComponent(factory);
            // Set the 'task' input on the component to the current task
            componentRef.instance.actions = this.task.actions;
        }
        this.actionsDisplayComponent = this.task.actionsDisplayComponent;
    }
    /**
     * @return {?}
     */
    updateBodyComponent() {
        // Clear the existing body component, if one exists
        this.bodyViewContainer.clear();
        // Insert a new body component, if one is given
        if (this.task.bodyComponent) {
            /** @type {?} */
            const factory = this.resolver.resolveComponentFactory(this.task.bodyComponent);
            /** @type {?} */
            const componentRef = this.bodyViewContainer.createComponent(factory);
            // Set the 'task' input on the component to the current task
            componentRef.instance.task = this.task;
        }
        // Save a reference to the current body component class
        this.bodyComponent = this.task.bodyComponent;
    }
}
CyberUiMinimalTaskExpansionPanelComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-minimal-task-expansion-panel',
                template: "<mat-expansion-panel>\n  <mat-expansion-panel-header expandedHeight=\"*\" collapsedHeight=\"*\">{{task.label}}</mat-expansion-panel-header>\n  <ng-template #body></ng-template>\n  <div class=\"cyber-ui-actions-display-component-container\">\n    <ng-template #actions></ng-template>\n  </div>\n  <mat-action-row\n      *ngIf=\"!task.actionsDisplayComponent && task.actions !== undefined && task.actions.length > 0\">\n    <button\n      mat-button\n      cyberUiActionContext\n      #actionContext=\"cyberUiActionContext\"\n      *ngFor=\"let action of task.actions\"\n      (click)=\"action.handler(actionContext.getContext())\"\n      >{{action.label}}</button>\n  </mat-action-row>\n</mat-expansion-panel>",
                styles: [".cyber-ui-actions-display-component-container:not(:empty){margin:0 -24px -16px}"]
            }] }
];
/** @nocollapse */
CyberUiMinimalTaskExpansionPanelComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver }
];
CyberUiMinimalTaskExpansionPanelComponent.propDecorators = {
    panel: [{ type: ViewChild, args: [MatExpansionPanel, { static: true },] }],
    bodyViewContainer: [{ type: ViewChild, args: ['body', { read: ViewContainerRef, static: true },] }],
    actionsViewContainer: [{ type: ViewChild, args: ['actions', { read: ViewContainerRef, static: true },] }],
    task: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiMinimalTaskExpansionPanelComponent.prototype.panel;
    /** @type {?} */
    CyberUiMinimalTaskExpansionPanelComponent.prototype.bodyViewContainer;
    /** @type {?} */
    CyberUiMinimalTaskExpansionPanelComponent.prototype.actionsViewContainer;
    /** @type {?} */
    CyberUiMinimalTaskExpansionPanelComponent.prototype.task;
    /** @type {?} */
    CyberUiMinimalTaskExpansionPanelComponent.prototype.bodyComponent;
    /** @type {?} */
    CyberUiMinimalTaskExpansionPanelComponent.prototype.actionsDisplayComponent;
    /**
     * @type {?}
     * @private
     */
    CyberUiMinimalTaskExpansionPanelComponent.prototype.resolver;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A relatively minimal display component for a set of tasks as a Material accordion
/**
 * @template TASK_T
 */
class CyberUiTaskAccordionComponent {
    // The tasks to display in the accordion
    /**
     * @param {?} tasks
     * @return {?}
     */
    set tasks(tasks) {
        if (this._tasks) {
            // Stop listening to the old tasks observable
            this.tasksSubscription.unsubscribe();
        }
        this._tasks = tasks;
        // Subscribe to the new tasks observable
        this.tasksSubscription = tasks.subscribe((/**
         * @param {?} _
         * @return {?}
         */
        _ => {
            // Draw attention to the new top suggestion
            // Needs to be done asynchronously to wait for the view to be updated
            // TODO Allow consumers to disable this functionality via a workflow setting
            setTimeout((/**
             * @return {?}
             */
            () => this.openFirstExpansionPanelIfExists()), 0);
        }));
    }
    // Opens the first expansion panel in the accordion, if it exists
    // The idea is to draw attention to the top suggestion whenever the suggestions change
    /**
     * @return {?}
     */
    openFirstExpansionPanelIfExists() {
        if (this.suggestionPanels !== undefined && this.suggestionPanels.length > 0) {
            this.suggestionPanels.first.panel.open();
        }
    }
}
CyberUiTaskAccordionComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-task-accordion',
                template: "<mat-accordion *ngIf=\"(_tasks | async) !== null\">\n  <cyber-ui-minimal-task-expansion-panel\n    *ngFor=\"let task of (_tasks | async)\"\n    [task]=\"task\">\n  </cyber-ui-minimal-task-expansion-panel>\n</mat-accordion>",
                styles: [":host{max-width:100%}"]
            }] }
];
CyberUiTaskAccordionComponent.propDecorators = {
    tasks: [{ type: Input }],
    suggestionPanels: [{ type: ViewChildren, args: [CyberUiMinimalTaskExpansionPanelComponent,] }]
};
if (false) {
    /** @type {?} */
    CyberUiTaskAccordionComponent.prototype._tasks;
    /** @type {?} */
    CyberUiTaskAccordionComponent.prototype.suggestionPanels;
    /** @type {?} */
    CyberUiTaskAccordionComponent.prototype.tasksSubscription;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiMinimalTaskExpansionPanelModule {
}
CyberUiMinimalTaskExpansionPanelModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatButtonModule,
                    MatExpansionModule,
                    CyberUiActionContextModule,
                ],
                declarations: [
                    CyberUiMinimalTaskExpansionPanelComponent,
                ],
                exports: [
                    CyberUiMinimalTaskExpansionPanelComponent,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiTaskAccordionModule {
}
CyberUiTaskAccordionModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatExpansionModule,
                    CyberUiMinimalTaskExpansionPanelModule,
                ],
                declarations: [
                    CyberUiTaskAccordionComponent,
                ],
                exports: [
                    CyberUiTaskAccordionComponent,
                ],
                entryComponents: [
                    // This component will often be dynamically generated and provided to cyber-ui-work-on-tasks-workflow
                    CyberUiTaskAccordionComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A relatively minimal display component for a task
// TODO The logic in this component is almost identical to that of minimal-expansion-panel and minimal-task-card; figure out a way to reduce the redundancy
/**
 * @template TASK_T
 */
class CyberUiMinimalTaskComponent {
    /**
     * @param {?} resolver
     */
    constructor(resolver) {
        this.resolver = resolver;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.task) {
            // If the task has changed, update the body component in the view container
            this.updateBodyComponent();
            this.updateActionsDisplayComponent();
        }
    }
    // There are some times we need to do work even if the task reference hasn't changed
    // e.g. if certain task properties have been mutated, we need to do some updates here
    /**
     * @return {?}
     */
    ngDoCheck() {
        if (this.actionsDisplayComponent !== this.task.actionsDisplayComponent) {
            this.updateActionsDisplayComponent();
        }
        else if (this.bodyComponent !== this.task.bodyComponent) {
            this.updateBodyComponent();
        }
    }
    /**
     * @return {?}
     */
    updateActionsDisplayComponent() {
        // Clear the existing actions display component, if one exists
        if (this.actionsViewContainer) {
            this.actionsViewContainer.clear();
        }
        if (this.task.actionsDisplayComponent) {
            /** @type {?} */
            const factory = this.resolver.resolveComponentFactory(this.task.actionsDisplayComponent);
            /** @type {?} */
            const componentRef = this.actionsViewContainer.createComponent(factory);
            // Set the 'task' input on the component to the current task
            componentRef.instance.actions = this.task.actions;
        }
        this.actionsDisplayComponent = this.task.actionsDisplayComponent;
    }
    /**
     * @return {?}
     */
    updateBodyComponent() {
        // Clear the existing body component, if one exists
        this.bodyViewContainer.clear();
        // Insert a new body component, if one is given
        if (this.task.bodyComponent) {
            /** @type {?} */
            const factory = this.resolver.resolveComponentFactory(this.task.bodyComponent);
            /** @type {?} */
            const componentRef = this.bodyViewContainer.createComponent(factory);
            // Set the 'task' input on the component to the current task
            componentRef.instance.task = this.task;
        }
        // Save a reference to the current body component class
        this.bodyComponent = this.task.bodyComponent;
    }
}
CyberUiMinimalTaskComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-minimal-task',
                template: "<div [class.cyber-ui-custom-actions-display-component]=\"task.actionsDisplayComponent\">\n  <h1>{{task.label}}</h1>\n  <ng-template #body></ng-template>\n  <ng-content></ng-content>\n  <div class=\"cyber-ui-actions-display-component-container\">\n    <ng-template #actions></ng-template>\n  </div>\n  <mat-chip-list *ngIf=\"!task.actionsDisplayComponent && task.actions !== undefined && task.actions.length > 0\">\n    <mat-chip\n      mat-button\n      cyberUiActionContext\n      #actionContext=\"cyberUiActionContext\"\n      *ngFor=\"let action of task.actions\"\n      (click)=\"action.handler(actionContext.getContext())\"\n      >{{action.label}}</mat-chip>\n  </mat-chip-list>\n</div>",
                styles: [":host{display:block;margin-bottom:32px}h1{color:var(--prominent-text-color)}"]
            }] }
];
/** @nocollapse */
CyberUiMinimalTaskComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver }
];
CyberUiMinimalTaskComponent.propDecorators = {
    bodyViewContainer: [{ type: ViewChild, args: ['body', { read: ViewContainerRef, static: true },] }],
    actionsViewContainer: [{ type: ViewChild, args: ['actions', { read: ViewContainerRef, static: true },] }],
    task: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiMinimalTaskComponent.prototype.bodyViewContainer;
    /** @type {?} */
    CyberUiMinimalTaskComponent.prototype.actionsViewContainer;
    /** @type {?} */
    CyberUiMinimalTaskComponent.prototype.task;
    /** @type {?} */
    CyberUiMinimalTaskComponent.prototype.bodyComponent;
    /** @type {?} */
    CyberUiMinimalTaskComponent.prototype.actionsDisplayComponent;
    /**
     * @type {?}
     * @private
     */
    CyberUiMinimalTaskComponent.prototype.resolver;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiMinimalTaskModule {
}
CyberUiMinimalTaskModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatChipsModule,
                    CyberUiActionContextModule,
                ],
                declarations: [
                    CyberUiMinimalTaskComponent,
                ],
                exports: [
                    CyberUiMinimalTaskComponent,
                ],
                entryComponents: [
                    CyberUiMinimalTaskComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiMinimalTaskListComponent {
}
CyberUiMinimalTaskListComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-minimal-task-list',
                template: "<mat-spinner *ngIf=\"(tasks | async) === null\"></mat-spinner>\n\n<ng-container *ngIf=\"(tasks | async) !== null\">\n  <cyber-ui-minimal-task\n    *ngFor=\"let task of (tasks | async)\"\n    [task]=\"task\">\n  </cyber-ui-minimal-task>\n</ng-container>",
                styles: ["cyber-ui-minimal-task-card:not(:last-child){margin-bottom:24px}:host mat-spinner{margin-top:64px}"]
            }] }
];
if (false) {
    /** @type {?} */
    CyberUiMinimalTaskListComponent.prototype.tasks;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiMinimalTaskListModule {
}
CyberUiMinimalTaskListModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    CyberUiMinimalTaskModule,
                    MatProgressSpinnerModule,
                ],
                declarations: [
                    CyberUiMinimalTaskListComponent,
                ],
                exports: [
                    CyberUiMinimalTaskListComponent,
                ],
                entryComponents: [
                    // This component will often be dynamically generated and provided to cyber-ui-work-on-tasks-workflow
                    CyberUiMinimalTaskListComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiMindfullyAttendToTopicTaskProviderModule {
}
CyberUiMindfullyAttendToTopicTaskProviderModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    CyberUiMindfullyAttendToTopicTaskProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiActionsPanelComponent {
    // Given an action, returns an object to be bound to the action button's ngStyle on hover
    // This logic uses the action's `color` hint to determine overflow menu actions' hover color
    /**
     * @param {?} action
     * @return {?}
     */
    getHoverStyle(action) {
        if (action.color) {
            return { color: action.color };
        }
        else {
            return {};
        }
    }
}
CyberUiActionsPanelComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-actions-panel',
                template: "<ng-container *ngIf=\"actions !== undefined && actions.length > 0\">\n  <button\n    mat-flat-button\n    cyberUiActionContext\n    #actionContext=\"cyberUiActionContext\"\n    *ngFor=\"let action of actions | slice:0:3\"\n    (click)=\"action.handler(actionContext.getContext())\"\n    [style.backgroundColor]=\"action.color\"\n    [style.color]=\"'#fff'\"\n    [style.flexGrow]=\"action.prominence || 1\"\n  >\n    <mat-icon *ngFor=\"let iconName of action.iconNames\">{{iconName}}</mat-icon>\n    {{action.label}}\n  </button>\n  <ng-container *ngIf=\"actions.length > 3\">\n    <button\n      mat-button\n      [matMenuTriggerFor]=\"moreActionsMenu\"\n      aria-label=\"More actions button\"\n      class=\"more-actions-button\"\n    >\n      MORE\n      <mat-icon >more_vert</mat-icon>\n    </button>\n    <mat-menu #moreActionsMenu=\"matMenu\">\n        <button\n            mat-menu-item\n            *ngFor=\"let action of actions | slice:3\"\n            cyberUiActionContext\n            #actionContext=\"cyberUiActionContext\"\n            (click)=\"action.handler(actionContext.getContext())\"\n            [ngStyle]=\"action._tmpNgStyle\"\n            (mouseover)=\"action._tmpNgStyle = getHoverStyle(action)\"\n            (mouseout)=\"action._tmpNgStyle = undefined\">\n          <mat-icon\n              *ngFor=\"let iconName of action.iconNames\"\n              [ngStyle]=\"action._tmpNgStyle\">{{iconName}}</mat-icon>\n          <span>{{action.label}}</span>\n        </button>\n    </mat-menu>\n  </ng-container>\n\n</ng-container>",
                styles: [":host{display:flex;flex-wrap:wrap}:host button{flex-basis:auto;flex-shrink:0;max-width:100%;white-space:pre-wrap;flex-grow:1;padding:8px 16px;border-radius:0}:host button.more-actions-button{min-width:auto}@media (max-width:599px){:host{flex-direction:column;flex-shrink:0}:host button{flex-basis:auto}}"]
            }] }
];
CyberUiActionsPanelComponent.propDecorators = {
    actions: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiActionsPanelComponent.prototype.actions;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiActionsPanelModule {
}
CyberUiActionsPanelModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatButtonModule,
                    MatIconModule,
                    MatMenuModule,
                    CyberUiActionContextModule,
                ],
                declarations: [
                    CyberUiActionsPanelComponent,
                ],
                exports: [
                    CyberUiActionsPanelComponent,
                ],
                entryComponents: [
                    CyberUiActionsPanelComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiResponseChipsComponent {
}
CyberUiResponseChipsComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-response-chips',
                template: "<mat-chip-list *ngIf=\"actions !== undefined && actions.length > 0\">\n  <mat-chip\n    mat-button\n    cyberUiActionContext\n    #actionContext=\"cyberUiActionContext\"\n    *ngFor=\"let action of actions\"\n    (click)=\"action.handler(actionContext.getContext())\"\n    >\n      <mat-icon\n        *ngFor=\"let iconName of action.iconNames\"\n        [style.color]=\"action.color\"\n        >{{iconName}}</mat-icon>\n      <ng-container *ngIf=\"action.iconNames\">&nbsp;</ng-container>\n      {{action.label}}\n  </mat-chip>\n</mat-chip-list>\n",
                styles: [".mat-chip.mat-standard-chip{background:0 0;border:1px solid var(--low-opacity-color);cursor:pointer;color:var(--text-color)}:host-context(cyber-ui-minimal-task-card) mat-chip-list{display:block;padding:16px}:host-context(cyber-ui-minimal-task-card) mat-chip-list .mat-chip.mat-standard-chip{color:var(--light-background-text-color);border-color:var(--light-background-low-opacity-color)}"]
            }] }
];
CyberUiResponseChipsComponent.propDecorators = {
    actions: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiResponseChipsComponent.prototype.actions;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiResponseChipsModule {
}
CyberUiResponseChipsModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    CyberUiActionContextModule,
                    MatChipsModule,
                    MatIconModule$1,
                ],
                declarations: [
                    CyberUiResponseChipsComponent,
                ],
                exports: [
                    CyberUiResponseChipsComponent,
                ],
                entryComponents: [
                    CyberUiResponseChipsComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template SETTINGS_T
 */
class CyberUiSettingsService {
    constructor() {
        // A replay subject containing the current settings object
        this.settings = new ReplaySubject(1);
    }
    /**
     * @param {?} settings
     * @return {?}
     */
    onChange(settings) {
        this.settings.next(settings);
    }
    // Returns a stream of settings optionally narrowed down by any provided property names
    // For example, listen('foo', 'bar') would provide a stream of settings.foo.bar values
    /**
     * @param {...?} propertyNames
     * @return {?}
     */
    listen(...propertyNames) {
        return this.settings.pipe(map((/**
         * @param {?} settings
         * @return {?}
         */
        settings => {
            for (let propertyName of propertyNames) {
                settings = settings[propertyName];
            }
            return settings;
        })));
    }
}
CyberUiSettingsService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ CyberUiSettingsService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiSettingsService_Factory() { return new CyberUiSettingsService(); }, token: CyberUiSettingsService, providedIn: "root" });
if (false) {
    /** @type {?} */
    CyberUiSettingsService.prototype.settings;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template SETTINGS_T
 */
class CyberUiSettingsDialogComponent {
    /**
     * @param {?} data
     * @param {?} settingsService
     */
    constructor(data, settingsService) {
        this.data = data;
        this.settingsService = settingsService;
        this.changes = new EventEmitter();
        this.config = data.config;
        this.settings = data.model;
        this.changes.subscribe((/**
         * @param {?} settings
         * @return {?}
         */
        settings => {
            this.settingsService.onChange(settings);
        }));
    }
    /**
     * @return {?}
     */
    onChange() {
        this.changes.emit(this.settings);
    }
}
CyberUiSettingsDialogComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-settings-dialog',
                template: "<div mat-dialog-title>{{config.label}}</div>\n<mat-dialog-content>\n  <cyber-ui-settings-section-items\n      [items]=\"config.items\"\n      [model]=\"settings\"\n      (changes)=\"onChange()\">\n  </cyber-ui-settings-section-items>\n</mat-dialog-content>\n"
            }] }
];
/** @nocollapse */
CyberUiSettingsDialogComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [MAT_DIALOG_DATA,] }] },
    { type: CyberUiSettingsService }
];
if (false) {
    /** @type {?} */
    CyberUiSettingsDialogComponent.prototype.config;
    /** @type {?} */
    CyberUiSettingsDialogComponent.prototype.settings;
    /** @type {?} */
    CyberUiSettingsDialogComponent.prototype.changes;
    /** @type {?} */
    CyberUiSettingsDialogComponent.prototype.data;
    /** @type {?} */
    CyberUiSettingsDialogComponent.prototype.settingsService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template SETTINGS_T
 */
class CyberUiSettingsDialogService {
    /**
     * @param {?} matDialog
     */
    constructor(matDialog) {
        this.matDialog = matDialog;
    }
    // Opens a settings dialog, and returns a Promise which will emit the result
    /**
     * @param {?} model
     * @param {?} config
     * @return {?}
     */
    open(
    // The settings data model object to mutate
    model, 
    // The settings config, governing layout of the settings
    config) {
        return this.matDialog.open(CyberUiSettingsDialogComponent, {
            data: { model, config }
        }).afterClosed().toPromise();
    }
}
CyberUiSettingsDialogService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
CyberUiSettingsDialogService.ctorParameters = () => [
    { type: MatDialog }
];
/** @nocollapse */ CyberUiSettingsDialogService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiSettingsDialogService_Factory() { return new CyberUiSettingsDialogService(ɵɵinject(MatDialog)); }, token: CyberUiSettingsDialogService, providedIn: "root" });
if (false) {
    /** @type {?} */
    CyberUiSettingsDialogService.prototype.matDialog;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template SETTINGS_T
 */
class CyberUiSettingsSectionItemsComponent {
    constructor() {
        this.changes = new EventEmitter();
    }
    /**
     * @return {?}
     */
    onChange() {
        this.changes.emit();
    }
}
CyberUiSettingsSectionItemsComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-settings-section-items',
                template: "<!-- If a label is provided, a legend element will be used, so wrap the output in a fieldset -->\n<fieldset *ngIf=\"label; else unwrapped\">\n  <ng-container *ngTemplateOutlet=\"unwrapped\"></ng-container>\n</fieldset>\n\n<ng-template #unwrapped>\n  <legend *ngIf=\"label\" class=\"settings-section-label\">{{label}}</legend>\n  <ng-container *ngIf=\"items\">\n    <ng-container *ngFor=\"let item of items\">\n      <cyber-ui-settings-section-items\n          *ngIf=\"item.items\"\n          [items]=\"item.items\"\n          [label]=\"item.label\"\n          [model]=\"model\"\n          (changes)=\"onChange()\">\n      </cyber-ui-settings-section-items>\n      <cyber-ui-form-fields\n          *ngIf=\"item.config\"\n          [field]=\"item\"\n          [model]=\"model\"\n          (change)=\"onChange()\">\n      </cyber-ui-form-fields>\n    </ng-container>\n  </ng-container>\n</ng-template>"
            }] }
];
CyberUiSettingsSectionItemsComponent.propDecorators = {
    label: [{ type: Input }],
    items: [{ type: Input }],
    model: [{ type: Input }],
    changes: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    CyberUiSettingsSectionItemsComponent.prototype.label;
    /** @type {?} */
    CyberUiSettingsSectionItemsComponent.prototype.items;
    /** @type {?} */
    CyberUiSettingsSectionItemsComponent.prototype.model;
    /** @type {?} */
    CyberUiSettingsSectionItemsComponent.prototype.changes;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiSettingsDialogModule {
}
CyberUiSettingsDialogModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatButtonModule,
                    MatDialogModule,
                    MatListModule,
                    CyberUiFormFieldsModule,
                ],
                declarations: [
                    CyberUiSettingsDialogComponent,
                    CyberUiSettingsSectionItemsComponent,
                ],
                exports: [
                    CyberUiSettingsDialogComponent,
                ],
                entryComponents: [
                    CyberUiSettingsDialogComponent,
                ],
                providers: [
                    CyberUiSettingsDialogService,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiSnoozeReasonCollectionDialogComponent {
    /**
     * @param {?} dialogRef
     * @param {?} data
     */
    constructor(dialogRef, data) {
        this.dialogRef = dialogRef;
        this.data = data;
        // TODO Once we have a reference to the snoozed entity's primary characterization
        // (e.g. 'task', 'project', etc.) update the title below to use
        // "Why did you snooze this {{characterization}}?"
        this.title = 'Why did you snooze this?';
        this.reasons = data.reasons;
    }
}
CyberUiSnoozeReasonCollectionDialogComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-snooze-reason-collection-dialog',
                template: "<div mat-dialog-title>{{title}}</div>\n<mat-dialog-content>\n  <mat-nav-list role=\"list\">\n    <a mat-list-item *ngFor=\"let reason of reasons\" (click)=\"dialogRef.close(reason)\">{{reason}}</a>\n  </mat-nav-list>\n</mat-dialog-content>\n",
                styles: [""]
            }] }
];
/** @nocollapse */
CyberUiSnoozeReasonCollectionDialogComponent.ctorParameters = () => [
    { type: MatDialogRef },
    { type: undefined, decorators: [{ type: Inject, args: [MAT_DIALOG_DATA,] }] }
];
if (false) {
    /** @type {?} */
    CyberUiSnoozeReasonCollectionDialogComponent.prototype.title;
    /** @type {?} */
    CyberUiSnoozeReasonCollectionDialogComponent.prototype.reasons;
    /** @type {?} */
    CyberUiSnoozeReasonCollectionDialogComponent.prototype.dialogRef;
    /** @type {?} */
    CyberUiSnoozeReasonCollectionDialogComponent.prototype.data;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SnoozeReason {
    /**
     * @param {?} label
     * @param {?=} children
     */
    constructor(label, children = []) {
        this.label = label;
        this.children = children;
    }
}
if (false) {
    /** @type {?} */
    SnoozeReason.prototype.label;
    /** @type {?} */
    SnoozeReason.prototype.children;
}
// This ontology is in progress and subject to change over time
// When storing the user's reason for snoozing something, it's recommended
// that consumering store the snooze reason label itself rather than any kind
// of index or reference into this object, since the particulars of this object
// could change over time.
/** @type {?} */
const CYBER_UI_SNOOZE_REASON_ONTOLOGY = [
    new SnoozeReason(
    // This reason indicates the task needs to be further refined
    'It needs to be refined', [
        new SnoozeReason(
        // This reason indicates the task would benefit from further decomposition
        "This task needs to be decomposed into subtasks"),
        new SnoozeReason(
        // This reason indicates that the path to completion of the task in non-obvious and would benefit from creation of a plan
        "This task needs a plan"),
    ]),
    new SnoozeReason(
    // This reason indicates that the reason was circumstantial rather
    // than due to an intrinsic property of the task
    "It's circumstantially suboptimal"),
    new SnoozeReason(
    // This indicates that the snoozed item was not as important as it
    // was deemed by the system, and should be demoted in the suggestions
    "It's a bad suggestion")
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Note: this will probably eventually live somewhere else
const ɵ0$1 = /**
 * @param {?} reason
 * @return {?}
 */
reason => reason.label;
/** @type {?} */
const DEFAULT_SNOOZE_REASONS = CYBER_UI_SNOOZE_REASON_ONTOLOGY.map((ɵ0$1));
class CyberUiSnoozeReasonCollectionDialogService {
    /**
     * @param {?} matDialog
     */
    constructor(matDialog) {
        this.matDialog = matDialog;
    }
    // Opens a snooze reason collection dialog, and returns a Promise which will emit the result
    /**
     * @param {?=} reasons
     * @return {?}
     */
    open(reasons = DEFAULT_SNOOZE_REASONS) {
        return this.matDialog.open(CyberUiSnoozeReasonCollectionDialogComponent, {
            data: { reasons },
        }).afterClosed().toPromise();
    }
}
CyberUiSnoozeReasonCollectionDialogService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root',
            },] }
];
/** @nocollapse */
CyberUiSnoozeReasonCollectionDialogService.ctorParameters = () => [
    { type: MatDialog }
];
/** @nocollapse */ CyberUiSnoozeReasonCollectionDialogService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiSnoozeReasonCollectionDialogService_Factory() { return new CyberUiSnoozeReasonCollectionDialogService(ɵɵinject(MatDialog)); }, token: CyberUiSnoozeReasonCollectionDialogService, providedIn: "root" });
if (false) {
    /** @type {?} */
    CyberUiSnoozeReasonCollectionDialogService.prototype.matDialog;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiSnoozeReasonCollectionDialogModule {
}
CyberUiSnoozeReasonCollectionDialogModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatButtonModule,
                    MatDialogModule,
                    MatListModule,
                ],
                declarations: [
                    CyberUiSnoozeReasonCollectionDialogComponent,
                ],
                exports: [
                    CyberUiSnoozeReasonCollectionDialogComponent,
                ],
                entryComponents: [
                    CyberUiSnoozeReasonCollectionDialogComponent,
                ],
                providers: [
                    CyberUiSnoozeReasonCollectionDialogService,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// By default, a filled circle is used as the displayed glyph
/** @type {?} */
const CYBER_UI_STATUS_INDICATOR_DEFAULT_GLPYH = '●';
// By default, 'unknown' is used as the status string
/** @type {?} */
const CYBER_UI_STATUS_INDICATOR_DEFAULT_STATUS = 'unknown';
class CyberUiStatusIndicatorState {
    /**
     * @param {?} options
     */
    constructor(options) {
        this.color = options.color;
        this.glyph = this.getGlyph(options);
        this.status = this.getStatus(options);
        this.tooltipText = options.tooltipText;
    }
    /**
     * @param {?} options
     * @return {?}
     */
    getGlyph(options) {
        return options.glyph || CYBER_UI_STATUS_INDICATOR_DEFAULT_GLPYH;
    }
    /**
     * @param {?} options
     * @return {?}
     */
    getStatus(options) {
        return options.status || CYBER_UI_STATUS_INDICATOR_DEFAULT_STATUS;
    }
}
if (false) {
    /** @type {?} */
    CyberUiStatusIndicatorState.prototype.color;
    /** @type {?} */
    CyberUiStatusIndicatorState.prototype.glyph;
    /** @type {?} */
    CyberUiStatusIndicatorState.prototype.status;
    /** @type {?} */
    CyberUiStatusIndicatorState.prototype.tooltipText;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiStatusIndicatorComponent {
    /**
     * @return {?}
     */
    get status() {
        return this.state && this.state.status;
    }
}
CyberUiStatusIndicatorComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-status-indicator',
                template: "<span\n  *ngIf=\"state\"\n  [style.color]=\"state.color\"\n  [matTooltip]=\"state.tooltipText\"\n>{{state.glyph}}</span>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [":host{display:block}span{font-size:1.25em}"]
            }] }
];
CyberUiStatusIndicatorComponent.propDecorators = {
    state: [{ type: Input }],
    status: [{ type: HostBinding, args: ['attr.status',] }]
};
if (false) {
    /** @type {?} */
    CyberUiStatusIndicatorComponent.prototype.state;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiStatusIndicatorModule {
}
CyberUiStatusIndicatorModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatTooltipModule,
                ],
                declarations: [
                    CyberUiStatusIndicatorComponent,
                ],
                exports: [
                    CyberUiStatusIndicatorComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Avoid "Cannot call a namespace ('tinycolor')" compilation error
// see e.g. https://github.com/jvandemo/generator-angular2-library/issues/221
/** @type {?} */
const tinycolor = tinycolor_;
// Static theme constants
/** @type {?} */
const DARK_THEME_TEXT_COLOR = 'rgba(255, 255, 255, 0.87)';
/** @type {?} */
const LIGHT_THEME_TEXT_COLOR = 'rgba(0, 0, 0, 0.87)';
/** @type {?} */
const DARK_THEME_SUBHEADER_COLOR = 'rgba(255, 255, 255, 0.54)';
/** @type {?} */
const LIGHT_THEME_SUBHEADER_COLOR = 'rgba(0, 0, 0, 0.54)';
/** @type {?} */
const DARK_THEME_LOW_OPACITY_COLOR = 'rgba(255, 255, 255, 0.42)';
/** @type {?} */
const LIGHT_THEME_LOW_OPACITY_COLOR = 'rgba(0, 0, 0, 0.42)';
class CyberUiThemeService {
    /**
     * @param {?} settingsService
     */
    constructor(settingsService) {
        this.settingsService = settingsService;
        this.currentThemeIds = new ReplaySubject(1);
        this.appBackgroundColor = new ReplaySubject(1);
        // The current text color to use for subheaders in Material lists
        this.matListSubheaderColor = new ReplaySubject(1);
        this.lowOpacityColor = new ReplaySubject(1);
        // The current text color to use in typical body text on the page
        // As currently implemented, opacity is diminished to reduce contrast
        // In situations where a solid, prominent foreground text color is desired,
        // use prominentTextColor instead
        this.textColor = new ReplaySubject(1);
        // A more prominent foreground text color appropriate given the current background
        // The current implementation preserves full opacity
        this.prominentTextColor = new ReplaySubject(1);
        // The primary color to use for links
        this.primaryLinkColor = new ReplaySubject(1);
        // Keep currentThemeId up to date
        this.currentThemeIds.subscribe((/**
         * @param {?} id
         * @return {?}
         */
        id => this.currentThemeId = id));
        // Subscribe to settings updates
        settingsService.listen('theme').subscribe((/**
         * @param {?} themeId
         * @return {?}
         */
        themeId => {
            if (themeId !== this.currentThemeId) {
                this.currentThemeIds.next(themeId);
            }
        }));
        // Keep cached properties up to date on settings changes
        settingsService.listen().subscribe((/**
         * @param {?} settings
         * @return {?}
         */
        settings => this.recalculateCachedProperties(settings)));
        // Update theme-related CSS custom properties
        this.appBackgroundColor.subscribe((/**
         * @param {?} color
         * @return {?}
         */
        color => this.setCssCustomProperty('--app-background-color', color)));
        this.textColor.subscribe((/**
         * @param {?} color
         * @return {?}
         */
        color => this.setCssCustomProperty('--text-color', color)));
        this.lowOpacityColor.subscribe((/**
         * @param {?} color
         * @return {?}
         */
        color => this.setCssCustomProperty('--low-opacity-color', color)));
        this.matListSubheaderColor.subscribe((/**
         * @param {?} color
         * @return {?}
         */
        color => this.setCssCustomProperty('--mat-list-subheader-color', color)));
        this.primaryLinkColor.subscribe((/**
         * @param {?} color
         * @return {?}
         */
        color => this.setCssCustomProperty('--primary-link-color', color)));
        this.prominentTextColor.subscribe((/**
         * @param {?} color
         * @return {?}
         */
        color => this.setCssCustomProperty('--prominent-text-color', color)));
        // Set static CSS custom properties
        this.setCssCustomProperty('--light-background-text-color', LIGHT_THEME_TEXT_COLOR);
        this.setCssCustomProperty('--light-background-subheader-color', LIGHT_THEME_SUBHEADER_COLOR);
        this.setCssCustomProperty('--light-background-low-opacity-color', LIGHT_THEME_LOW_OPACITY_COLOR);
    }
    /**
     * @param {?} settings
     * @return {?}
     */
    recalculateCachedProperties(settings) {
        this.topToolbarBackgroundColor = this.getTopToolbarBackgroundColor(settings);
        this.appBackground = this.getAppBackground(settings);
        this.appBackgroundColor.next(this.getAppBackground(settings));
        this.lowOpacityColor.next(this.getLowOpacityColor(settings));
        this.matListSubheaderColor.next(this.getMatListSubheaderColor(settings));
        this.textColor.next(this.getTextColor(settings));
        this.prominentTextColor.next(this.getProminentTextColor(settings));
        this.primaryLinkColor.next(this.getPrimaryLinkColor(settings));
    }
    /**
     * @param {?} settings
     * @return {?}
     */
    getMatListSubheaderColor(settings) {
        // TODO There's some more work to do here, but as a starting point:
        // if the background color is dark, use white; if it's bright, use black.
        /** @type {?} */
        const color = tinycolor(this.getAppBackground(settings));
        if (color.getBrightness() < 127) {
            return DARK_THEME_SUBHEADER_COLOR;
        }
        else {
            return LIGHT_THEME_SUBHEADER_COLOR;
        }
    }
    /**
     * @param {?} settings
     * @return {?}
     */
    getLowOpacityColor(settings) {
        // TODO There's some more work to do here, but as a starting point:
        // if the background color is dark, use white; if it's bright, use black.
        /** @type {?} */
        const color = tinycolor(this.getAppBackground(settings));
        if (color.getBrightness() < 127) {
            return DARK_THEME_LOW_OPACITY_COLOR;
        }
        else {
            return LIGHT_THEME_LOW_OPACITY_COLOR;
        }
    }
    /**
     * @param {?} settings
     * @return {?}
     */
    getPrimaryLinkColor(settings) {
        /** @type {?} */
        const appBackgroundColor = tinycolor(this.getAppBackground(settings));
        /** @type {?} */
        const topToolbarBackgroundColor = tinycolor(this.getTopToolbarBackgroundColor(settings));
        if (appBackgroundColor.isLight() && topToolbarBackgroundColor.isDark()) {
            return topToolbarBackgroundColor.darken(10);
        }
        else if (appBackgroundColor.isLight() && topToolbarBackgroundColor.isLight()) {
            return 'blue';
        }
        else {
            // The app background color is dark
            return appBackgroundColor.lighten(25).spin(10);
        }
    }
    /**
     * @param {?} settings
     * @return {?}
     */
    getProminentTextColor(settings) {
        // TODO There's some more work to do here, but as a starting point:
        // if the background color is dark, use white; if it's bright, use black.
        /** @type {?} */
        const color = tinycolor(this.getAppBackground(settings));
        if (color.getBrightness() < 127) {
            return 'white';
        }
        else {
            return 'black';
        }
    }
    /**
     * @param {?} settings
     * @return {?}
     */
    getTextColor(settings) {
        // TODO There's some more work to do here, but as a starting point:
        // if the background color is dark, use white; if it's bright, use black.
        /** @type {?} */
        const color = tinycolor(this.getAppBackground(settings));
        if (color.getBrightness() < 127) {
            return DARK_THEME_TEXT_COLOR;
        }
        else {
            return LIGHT_THEME_TEXT_COLOR;
        }
    }
    /**
     * @param {?} settings
     * @return {?}
     */
    getTopToolbarBackgroundColor(settings) {
        return settings.topToolbarDefaultBackgroundColor;
    }
    /**
     * @param {?} settings
     * @return {?}
     */
    getAppBackground(settings) {
        if (settings.matchAppAndToolbarBackground) {
            return this.topToolbarBackgroundColor;
        }
        else {
            // TODO This should be configurable
            return 'white';
        }
    }
    // Utility method to pin the active theme class to the body element
    /**
     * @param {?} document
     * @param {?} renderer
     * @return {?}
     */
    pinActiveThemeClassOnBodyElement(document, renderer) {
        /** @type {?} */
        let previousThemeId = undefined;
        this.currentThemeIds.subscribe((/**
         * @param {?} themeId
         * @return {?}
         */
        themeId => {
            // Remove the old theme class
            if (previousThemeId) {
                renderer.removeClass(document.body, `cyber-ui-${previousThemeId}-theme`);
            }
            // Add the new theme class
            renderer.addClass(document.body, `cyber-ui-${themeId}-theme`);
            // Update the reference to the previous theme ID
            previousThemeId = themeId;
        }));
    }
    /**
     * @param {?} propertyName
     * @param {?} value
     * @return {?}
     */
    setCssCustomProperty(propertyName, value) {
        document.documentElement.style.setProperty(propertyName, value);
    }
}
CyberUiThemeService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
CyberUiThemeService.ctorParameters = () => [
    { type: CyberUiSettingsService }
];
/** @nocollapse */ CyberUiThemeService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiThemeService_Factory() { return new CyberUiThemeService(ɵɵinject(CyberUiSettingsService)); }, token: CyberUiThemeService, providedIn: "root" });
if (false) {
    /** @type {?} */
    CyberUiThemeService.prototype.currentThemeIds;
    /** @type {?} */
    CyberUiThemeService.prototype.currentThemeId;
    /** @type {?} */
    CyberUiThemeService.prototype.topToolbarBackgroundColor;
    /** @type {?} */
    CyberUiThemeService.prototype.appBackground;
    /** @type {?} */
    CyberUiThemeService.prototype.appBackgroundColor;
    /** @type {?} */
    CyberUiThemeService.prototype.matListSubheaderColor;
    /** @type {?} */
    CyberUiThemeService.prototype.lowOpacityColor;
    /** @type {?} */
    CyberUiThemeService.prototype.textColor;
    /** @type {?} */
    CyberUiThemeService.prototype.prominentTextColor;
    /** @type {?} */
    CyberUiThemeService.prototype.primaryLinkColor;
    /** @type {?} */
    CyberUiThemeService.prototype.settingsService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiThemeModule {
}
CyberUiThemeModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    CyberUiThemeService,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const MATCH_APP_AND_TOOLBAR_BACKGROUND = new BooleanField({
    label: 'Match app and toolbar background?',
    propertyName: 'matchAppAndToolbarBackground',
    helpText: 'If true, the app background will be configured to match that of the top toolbar, removing the visual boundary between them.',
});
/** @type {?} */
const TOP_TOOLBAR_DEFAULT_BACKGROUND_COLOR = new TextField({
    label: 'Top toolbar default background color',
    propertyName: 'topToolbarDefaultBackgroundColor',
});
/** @type {?} */
const TOP_TOOLBAR_GRADIENT = new BooleanField({
    label: 'Top toolbar gradient?',
    propertyName: 'topToolbarGradientActive',
});
/** @type {?} */
const TOP_TOOLBAR_DROP_SHADOW = new BooleanField({
    label: 'Top toolbar drop shadow?',
    propertyName: 'topToolbarDropShadowActive',
});
/** @type {?} */
const CYBER_UI_MATERIAL_THEME_SETTINGS = [
    TOP_TOOLBAR_DEFAULT_BACKGROUND_COLOR,
    MATCH_APP_AND_TOOLBAR_BACKGROUND,
    TOP_TOOLBAR_GRADIENT,
    TOP_TOOLBAR_DROP_SHADOW,
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const WORKFLOW_SETTINGS_SERVICE = new InjectionToken('WorkflowSettingsService');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const CYBER_UI_ADD_THING_WORKFLOW_DISPLAY_COMPONENT = new InjectionToken('CyberUiAddThingWorkflowDisplayComponent');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiAddThingWorkflowService {
    constructor() {
        // Stream of exit events indicating when the workflow is finished
        this.exits = new EventEmitter();
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    exitWorkflow(options = {}) {
        this.exits.emit(options);
    }
}
CyberUiAddThingWorkflowService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */ CyberUiAddThingWorkflowService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiAddThingWorkflowService_Factory() { return new CyberUiAddThingWorkflowService(); }, token: CyberUiAddThingWorkflowService, providedIn: "root" });
if (false) {
    /** @type {?} */
    CyberUiAddThingWorkflowService.prototype.exits;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO At some point, refactor this component to allow for dynamic changing of the display component
class CyberUiAddThingWorkflowComponent {
    /**
     * @param {?} modelService
     * @param {?} displayComponent
     * @param {?} service
     * @param {?} componentFactoryResolver
     * @param {?} viewContainerRef
     */
    constructor(modelService, displayComponent, service, componentFactoryResolver, viewContainerRef) {
        this.modelService = modelService;
        this.displayComponent = displayComponent;
        this.service = service;
        this.componentFactoryResolver = componentFactoryResolver;
        this.viewContainerRef = viewContainerRef;
        // Instantiate the display component
        /** @type {?} */
        const componentFactory = this.componentFactoryResolver.resolveComponentFactory(displayComponent);
        this.displayComponentRef = this.viewContainerRef.createComponent(componentFactory);
        // Get a new empty model and hook it up to the display component
        this.displayComponentRef.instance.model = modelService.getNewEmptyModel();
        // Handle save and exit events from the display component
        this.displayComponentRef.instance.saves.subscribe((/**
         * @param {?} model
         * @return {?}
         */
        model => this.handleSave(model)));
        this.displayComponentRef.instance.exits.subscribe((/**
         * @return {?}
         */
        () => this.service.exitWorkflow()));
    }
    /**
     * @param {?} model
     * @return {?}
     */
    handleSave(model) {
        model.save().then((/**
         * @return {?}
         */
        () => this.service.exitWorkflow({ model: model })), (/**
         * @param {?} error
         * @return {?}
         */
        error => console.error(error)));
    }
    // Discards the existing model and gets a new one
    /**
     * @return {?}
     */
    clearModel() {
        this.displayComponentRef.instance.model = this.modelService.getNewEmptyModel();
    }
}
CyberUiAddThingWorkflowComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-add-thing-workflow',
                template: ''
            }] }
];
/** @nocollapse */
CyberUiAddThingWorkflowComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [CYBER_UI_MODEL_SERVICE,] }] },
    { type: Type, decorators: [{ type: Inject, args: [CYBER_UI_ADD_THING_WORKFLOW_DISPLAY_COMPONENT,] }] },
    { type: CyberUiAddThingWorkflowService },
    { type: ComponentFactoryResolver },
    { type: ViewContainerRef }
];
if (false) {
    /** @type {?} */
    CyberUiAddThingWorkflowComponent.prototype.displayComponentRef;
    /** @type {?} */
    CyberUiAddThingWorkflowComponent.prototype.modelService;
    /** @type {?} */
    CyberUiAddThingWorkflowComponent.prototype.displayComponent;
    /** @type {?} */
    CyberUiAddThingWorkflowComponent.prototype.service;
    /** @type {?} */
    CyberUiAddThingWorkflowComponent.prototype.componentFactoryResolver;
    /** @type {?} */
    CyberUiAddThingWorkflowComponent.prototype.viewContainerRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiAddThingWorkflowModule {
}
CyberUiAddThingWorkflowModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    CyberUiAddThingWorkflowComponent,
                ],
                exports: [
                    CyberUiAddThingWorkflowComponent,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?=} options
 * @return {?}
 */
function getLabelField(options = {}) {
    return new TextField(Object.assign({
        label: 'Label',
        propertyName: 'label'
    }, options));
}
/** @type {?} */
const LABEL_FIELD = getLabelField();

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO Make this dynamic and developer-configurable
/** @type {?} */
const TEMPORARY_HARDCODED_FIELD_LIST = [
    getLabelField({ autofocus: true, saveOnEnter: true })
];
// A default display component implementation for add-thing-workflow
// Developers may elect to provide a substitute display component via the
// CYBER_UI_ADD_THING_WORKFLOW_DISPLAY_COMPONENT injection token
class CyberUiAddThingWorkflowDefaultDisplayComponent {
    /**
     * @param {?} themeService
     */
    constructor(themeService) {
        this.fields = TEMPORARY_HARDCODED_FIELD_LIST;
        // If saving, the event will contain a reference to the model to save
        this.saves = new EventEmitter();
        this.exits = new EventEmitter();
        // Change the text color based on the current theme
        themeService.textColor.subscribe((/**
         * @param {?} color
         * @return {?}
         */
        color => {
            this.color = color;
        }));
        themeService.prominentTextColor.subscribe((/**
         * @param {?} color
         * @return {?}
         */
        color => {
            this.prominentTextColor = color;
        }));
    }
}
CyberUiAddThingWorkflowDefaultDisplayComponent.decorators = [
    { type: Component, args: [{
                template: "<h2 [style.color]=\"prominentTextColor\">Add thing</h2>\n<cyber-ui-form-fields\n  [model]=\"model\"\n  [fields]=\"fields\"\n  (save)=\"saves.emit(model)\"></cyber-ui-form-fields>\n<button mat-button mat-flat-button color=\"primary\" (click)=\"saves.emit(model)\">SAVE</button>\n<button mat-button (click)=\"exits.emit()\">DISCARD</button>"
            }] }
];
/** @nocollapse */
CyberUiAddThingWorkflowDefaultDisplayComponent.ctorParameters = () => [
    { type: CyberUiThemeService }
];
CyberUiAddThingWorkflowDefaultDisplayComponent.propDecorators = {
    color: [{ type: HostBinding, args: ['style.color',] }]
};
if (false) {
    /** @type {?} */
    CyberUiAddThingWorkflowDefaultDisplayComponent.prototype.fields;
    /** @type {?} */
    CyberUiAddThingWorkflowDefaultDisplayComponent.prototype.model;
    /** @type {?} */
    CyberUiAddThingWorkflowDefaultDisplayComponent.prototype.saves;
    /** @type {?} */
    CyberUiAddThingWorkflowDefaultDisplayComponent.prototype.exits;
    /** @type {?} */
    CyberUiAddThingWorkflowDefaultDisplayComponent.prototype.color;
    /** @type {?} */
    CyberUiAddThingWorkflowDefaultDisplayComponent.prototype.prominentTextColor;
}
// It's assumed that any substitute display component will still conform to the following type where relevant
class CyberUiAddThingWorkflowDisplayComponent {
}
CyberUiAddThingWorkflowDisplayComponent.decorators = [
    { type: Component, args: [{
                template: ''
            }] }
];
if (false) {
    /** @type {?} */
    CyberUiAddThingWorkflowDisplayComponent.prototype.model;
    /** @type {?} */
    CyberUiAddThingWorkflowDisplayComponent.prototype.saves;
    /** @type {?} */
    CyberUiAddThingWorkflowDisplayComponent.prototype.exits;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiAddThingWorkflowDefaultDisplayModule {
}
CyberUiAddThingWorkflowDefaultDisplayModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    MatButtonModule,
                    CyberUiFormFieldsModule,
                ],
                declarations: [
                    // The first one exists only to provide a type, and should probably be moved elsewhere
                    CyberUiAddThingWorkflowDisplayComponent,
                    CyberUiAddThingWorkflowDefaultDisplayComponent,
                ],
                exports: [
                    CyberUiAddThingWorkflowDefaultDisplayComponent,
                ],
                entryComponents: [
                    CyberUiAddThingWorkflowDefaultDisplayComponent,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiWorkOnTasksWorkflowService {
    constructor() {
        this.reloadEvents = new EventEmitter();
    }
    // Force a reload of the suggestions
    /**
     * @return {?}
     */
    reloadSuggestions() {
        this.reloadEvents.emit();
    }
}
CyberUiWorkOnTasksWorkflowService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ CyberUiWorkOnTasksWorkflowService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CyberUiWorkOnTasksWorkflowService_Factory() { return new CyberUiWorkOnTasksWorkflowService(); }, token: CyberUiWorkOnTasksWorkflowService, providedIn: "root" });
if (false) {
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowService.prototype.reloadEvents;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// A workflow for working on tasks using an arbitrary task-list-rendering component
/**
 * @template TASK_T, GET_SUGGESTIONS_OPTIONS_T, GET_SUGGESTIONS_RESPONSE_T
 */
class CyberUiWorkOnTasksWorkflowComponent {
    /**
     * @param {?} taskSuggestionService
     * @param {?} workflowSettingsService
     * @param {?} resolver
     * @param {?} service
     */
    constructor(taskSuggestionService, workflowSettingsService, resolver, service) {
        this.taskSuggestionService = taskSuggestionService;
        this.workflowSettingsService = workflowSettingsService;
        this.resolver = resolver;
        this.service = service;
        // A subject containing the latest tasks (to pass to the display component)
        this.tasks = new ReplaySubject(1);
        // Get options to use during the initial request
        this.getSuggestionsOptions = workflowSettingsService.getGetSuggestionsOptions();
        // Set up the suggestions subscription, given the initial options
        this.loadSuggestions();
        // Handle workflow settings changes
        this.workflowSettingsService.changes.subscribe((/**
         * @param {?} settings
         * @return {?}
         */
        (settings) => {
            this.getSuggestionsOptions = settings;
            if (settings.reloadSuggestionsAfterWorkflowSettingsChange) {
                this.loadSuggestions();
            }
        }));
        // Subscribe to any reload events sent by the service
        // This allows consumers to request a reload of suggestions in response to e.g. user action
        service.reloadEvents.subscribe((/**
         * @return {?}
         */
        () => this.loadSuggestions()));
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.displayComponent) {
            // Clear the existing display component, if one exists
            this.viewContainer.clear();
            // Insert the new body component
            /** @type {?} */
            const factory = this.resolver.resolveComponentFactory(changes.displayComponent.currentValue);
            /** @type {?} */
            const componentRef = this.viewContainer.createComponent(factory);
            // Set the 'tasks' input on the component to the tasks subject
            componentRef.instance.tasks = this.tasks;
        }
    }
    // Subscribes to the suggestion service using the latest workflow settings
    /**
     * @return {?}
     */
    loadSuggestions() {
        // Unsubscribe from the previous subscription, if it exists
        if (this.getSuggestionsSubscription !== undefined && !this.getSuggestionsSubscription.closed) {
            this.getSuggestionsSubscription.unsubscribe();
        }
        this.getSuggestionsSubscription = this.taskSuggestionService
            .getSuggestions(this.getSuggestionsOptions)
            .subscribe((/**
         * @param {?} response
         * @return {?}
         */
        (response) => {
            // Replace the old response (or initial undefined value) with the new response
            this.suggestionsResponse = response;
            // Update the tasks subject
            this.tasks.next(response.suggestions.map((/**
             * @param {?} suggestion
             * @return {?}
             */
            suggestion => suggestion.task)));
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        // Clean up and unsubscribe from the task suggestion service, to avoid memory leaks
        if (this.getSuggestionsSubscription !== undefined && !this.getSuggestionsSubscription.closed) {
            this.getSuggestionsSubscription.unsubscribe();
        }
    }
}
CyberUiWorkOnTasksWorkflowComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-work-on-tasks-workflow',
                template: "<cyber-ui-filter-chips\n  *ngIf=\"filters\"\n  [filters]=\"filters\"\n  [settings]=\"filterSettings\"\n></cyber-ui-filter-chips>\n<ng-template #view></ng-template>\n"
            }] }
];
/** @nocollapse */
CyberUiWorkOnTasksWorkflowComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [TASK_SUGGESTION_SERVICE,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [WORKFLOW_SETTINGS_SERVICE,] }] },
    { type: ComponentFactoryResolver },
    { type: CyberUiWorkOnTasksWorkflowService }
];
CyberUiWorkOnTasksWorkflowComponent.propDecorators = {
    viewContainer: [{ type: ViewChild, args: ['view', { read: ViewContainerRef, static: true },] }],
    displayComponent: [{ type: Input }],
    filters: [{ type: Input }],
    filterSettings: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.getSuggestionsOptions;
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.suggestionsResponse;
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.getSuggestionsSubscription;
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.tasks;
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.viewContainer;
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.displayComponent;
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.filters;
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.filterSettings;
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.taskSuggestionService;
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.workflowSettingsService;
    /**
     * @type {?}
     * @private
     */
    CyberUiWorkOnTasksWorkflowComponent.prototype.resolver;
    /** @type {?} */
    CyberUiWorkOnTasksWorkflowComponent.prototype.service;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiChoiceFieldMenuComponent {
    constructor() {
        // A stream of clear events for the containing component to listen to
        this.clear = new EventEmitter();
    }
    // TODO Test that this method works whether the model is a POJO or a CyberUiInteractiveModel
    /**
     * @param {?} option
     * @return {?}
     */
    choose(option) {
        this.field.setModelProperty(this.model, this.field.config.propertyName, option.value);
    }
}
CyberUiChoiceFieldMenuComponent.decorators = [
    { type: Component, args: [{
                template: "<span #trigger [matMenuTriggerFor]=\"menu\"></span>\n<mat-menu #menu=\"matMenu\">\n  <!-- This menu is lazily rendered to avoid 'changed after it was checked' error-->\n  <ng-template matMenuContent>\n    <header mat-menu-item disabled>\n      <span class=\"mat-subheading-2\">{{field.config.label}}</span>\n      <button mat-icon-button (click)=\"clear.emit()\">\n        <mat-icon>close</mat-icon>\n      </button>\n    </header>\n    <div mat-menu-item\n        *ngFor=\"let option of field.config.options\"\n        (click)=\"choose(option)\">\n      <mat-radio-button\n        [checked]=\"field.getModelProperty(model, field.config.propertyName) === option.value\"\n      >{{option.label}}</mat-radio-button>\n    </div>\n  </ng-template>\n</mat-menu>\n",
                styles: ["header{color:#000!important;display:flex;align-items:stretch;overflow:visible}header>span.mat-subheading-2{flex-grow:1;padding-right:16px}header [mat-icon-button]{margin:-8px -16px 0 0}header [mat-icon-button] mat-icon{margin:0}"]
            }] }
];
CyberUiChoiceFieldMenuComponent.propDecorators = {
    field: [{ type: Input }],
    model: [{ type: Input }],
    clear: [{ type: Output }],
    trigger: [{ type: ViewChild, args: ['trigger', { read: MatMenuTrigger, static: true },] }]
};
if (false) {
    /** @type {?} */
    CyberUiChoiceFieldMenuComponent.prototype.field;
    /** @type {?} */
    CyberUiChoiceFieldMenuComponent.prototype.model;
    /** @type {?} */
    CyberUiChoiceFieldMenuComponent.prototype.clear;
    /** @type {?} */
    CyberUiChoiceFieldMenuComponent.prototype.trigger;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiChoiceFieldMenuModule {
}
CyberUiChoiceFieldMenuModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatButtonModule,
                    MatIconModule,
                    MatMenuModule,
                    MatRadioModule,
                ],
                declarations: [
                    CyberUiChoiceFieldMenuComponent,
                ],
                entryComponents: [
                    CyberUiChoiceFieldMenuComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Trailing icon names to re-use
/** @type {?} */
const TRAILING_ICON = {
    ARROW_DROP_DOWN: 'arrow_drop_down',
    CLOSE: 'close',
};
class CyberUiFilterChipsComponent {
    /**
     * @param {?} componentFactoryResolver
     * @param {?} themeService
     */
    constructor(componentFactoryResolver, themeService) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.themeService = themeService;
        // Using matListSubheaderColor since we want some opacity
        // TODO Utilize a better naming convention to make it more
        // clear what we're asking for from the theme service here
        themeService.textColor.subscribe((/**
         * @param {?} color
         * @return {?}
         */
        color => {
            this.color = color;
        }));
    }
    // Clears the filter
    /**
     * @param {?} field
     * @return {?}
     */
    clearFilter(field) {
        delete this.settings[field.config.propertyName];
    }
    /**
     * @param {?} field
     * @param {?} viewContainer
     * @return {?}
     */
    handleClick(field, viewContainer) {
        // Clear the previous menu, if one exists
        viewContainer.clear();
        // If the field is a choice field, bring up the choices in a menu
        if (field instanceof ChoiceField) {
            /** @type {?} */
            const factory = this.componentFactoryResolver.resolveComponentFactory(CyberUiChoiceFieldMenuComponent);
            /** @type {?} */
            const cmpRef = viewContainer.createComponent(factory);
            // Link the menu to the field
            cmpRef.instance.field = field;
            // Link up the menu to the settings
            cmpRef.instance.model = this.settings;
            // Subscribe to the first clear action taken from the menu, and clear the filter when that happens
            cmpRef.instance.clear.pipe(first()).subscribe((/**
             * @return {?}
             */
            () => {
                this.clearFilter(field);
                // Close the menu
                cmpRef.instance.trigger.closeMenu();
            }));
            // Trigger the menu
            // Needs to be done asynchronously to wait for the component's view to init
            setTimeout((/**
             * @return {?}
             */
            () => cmpRef.instance.trigger.openMenu()), 0);
        }
        else {
            // Log error / show snack about not knowing how to handle this type of field here yet
        }
    }
    /**
     * @param {?} field
     * @param {?} viewContainer
     * @param {?=} event
     * @return {?}
     */
    handleIconClick(field, viewContainer, event) {
        // If the field is currently active, clicking the icon clears the filter
        // Otherwise, it should have the same behavior as clicking the rest of the chip
        if (this.isFilterActive(field)) {
            this.clearFilter(field);
        }
        else {
            this.handleClick(field, viewContainer);
        }
        // Since the click was handled here, stop it from bubbling up
        if (event) {
            event.stopPropagation();
            ;
        }
        return false;
    }
    // Returns the label to use for this filter chip (dynamic based on the field's current value)
    /**
     * @param {?} field
     * @return {?}
     */
    getLabelForField(field) {
        if (this.settings[field.config.propertyName] !== undefined) {
            return `${field.config.label}: ${field.format(this.settings[field.config.propertyName])}`;
        }
        else {
            return field.config.label;
        }
    }
    // Returns whether this filter chip is considered active or not
    /**
     * @param {?} field
     * @return {?}
     */
    isFilterActive(field) {
        return this.settings[field.config.propertyName] !== undefined;
    }
    /**
     * @param {?} field
     * @return {?}
     */
    getTrailingIconName(field) {
        // If the filter is active, show the close icon
        // Otherwise, show the drop down icon
        if (this.isFilterActive(field)) {
            return TRAILING_ICON.CLOSE;
        }
        else {
            return TRAILING_ICON.ARROW_DROP_DOWN;
        }
    }
}
CyberUiFilterChipsComponent.decorators = [
    { type: Component, args: [{
                selector: 'cyber-ui-filter-chips',
                template: "<mat-chip-list [multiple]=\"true\">\n  <mat-chip\n    *ngFor=\"let field of filters\"\n    (click)=\"handleClick(field, directive.viewContainer)\"\n    cyberUiFilterChipDirective\n    #directive=\"cyberUiFilterChipDirective\"\n    color=\"primary\"\n    [selected]=\"isFilterActive(field)\"\n  >{{getLabelForField(field)}}<mat-icon\n    matChipTrailingIcon\n    (click)=\"handleIconClick(field, directive.viewContainer, $event)\"\n  >{{getTrailingIconName(field)}}</mat-icon></mat-chip>\n</mat-chip-list>",
                styles: [":host{display:flex;padding-bottom:1.75em;justify-content:flex-end}.mat-chip.mat-standard-chip{background:0 0;color:inherit;cursor:pointer}"]
            }] }
];
/** @nocollapse */
CyberUiFilterChipsComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: CyberUiThemeService }
];
CyberUiFilterChipsComponent.propDecorators = {
    filters: [{ type: Input }],
    settings: [{ type: Input }],
    color: [{ type: HostBinding, args: ['style.color',] }]
};
if (false) {
    /** @type {?} */
    CyberUiFilterChipsComponent.prototype.filters;
    /** @type {?} */
    CyberUiFilterChipsComponent.prototype.settings;
    /** @type {?} */
    CyberUiFilterChipsComponent.prototype.color;
    /** @type {?} */
    CyberUiFilterChipsComponent.prototype.componentFactoryResolver;
    /** @type {?} */
    CyberUiFilterChipsComponent.prototype.themeService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiFilterChipDirective {
    /**
     * @param {?} viewContainer
     */
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
}
CyberUiFilterChipDirective.decorators = [
    { type: Directive, args: [{
                selector: '[cyberUiFilterChipDirective]',
                exportAs: 'cyberUiFilterChipDirective',
            },] }
];
/** @nocollapse */
CyberUiFilterChipDirective.ctorParameters = () => [
    { type: ViewContainerRef }
];
if (false) {
    /** @type {?} */
    CyberUiFilterChipDirective.prototype.viewContainer;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiFilterChipsModule {
}
CyberUiFilterChipsModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatChipsModule$1,
                    MatIconModule,
                    CyberUiChoiceFieldMenuModule,
                ],
                declarations: [
                    CyberUiFilterChipDirective,
                    CyberUiFilterChipsComponent,
                ],
                exports: [
                    CyberUiFilterChipsComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiWorkOnTasksWorkflowModule {
}
CyberUiWorkOnTasksWorkflowModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    // TODO Don't depend on Material modules from this theme-agnostic workflow module
                    CyberUiFilterChipsModule,
                ],
                declarations: [
                    CyberUiWorkOnTasksWorkflowComponent,
                ],
                exports: [
                    CyberUiWorkOnTasksWorkflowComponent,
                ],
                providers: [
                    CyberUiWorkOnTasksWorkflowService,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Start with an instance of MatDialogConfig in order to inherit the Material defaults
/** @type {?} */
const CYBER_UI_MAT_DIALOG_DEFAULT_CONFIG = new MatDialogConfig();
// The `width` property functions in some ways as a maximum width, since it still works
// well on smaller screen sizes. `width` also serves as a width hint; this property is
// set, rather than 'maxWidth', in order for the dialog to actually take up this space
// when it is available.
CYBER_UI_MAT_DIALOG_DEFAULT_CONFIG.width = '1024px';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CyberUiTimeboxModule {
}
CyberUiTimeboxModule.decorators = [
    { type: NgModule, args: [{
                providers: [CyberUiTimeboxService],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { BooleanField, COMPLETION_CRITERIA_TEXT_FIELD, CYBER_UI_ADD_THING_WORKFLOW_DISPLAY_COMPONENT, CYBER_UI_MATERIAL_THEME_SETTINGS, CYBER_UI_MAT_DIALOG_DEFAULT_CONFIG, CYBER_UI_MINDFULLY_ATTEND_TO_TOPIC_TASK_PROVIDER_ID, CYBER_UI_MINDFULLY_ATTEND_TO_TOPIC_TASK_PROVIDER_LABEL, CYBER_UI_MODEL_SERVICE, CYBER_UI_SHOW_EDIT_DIALOG_CANCEL_ACTION, ChoiceField, CommaSeparatedListField, CyberUiActionContextModule, CyberUiActionsPanelComponent, CyberUiActionsPanelModule, CyberUiAddThingWorkflowComponent, CyberUiAddThingWorkflowDefaultDisplayComponent, CyberUiAddThingWorkflowDefaultDisplayModule, CyberUiAddThingWorkflowModule, CyberUiAddThingWorkflowService, CyberUiAttentionalAgendaDisplayModule, CyberUiAttentionalAgendaService, CyberUiAttentionalAgendaSnapshot, CyberUiAutofocusModule, CyberUiDelegationHistoryService, CyberUiDelegationMenuModule, CyberUiDelegationMenuService, CyberUiEditDialogComponent, CyberUiEditDialogModule, CyberUiEditDialogService, CyberUiFakeBackendModule, CyberUiFirestoreBackedModel, CyberUiFirestoreBackedModelService, CyberUiFirestoreBackend, CyberUiFirestoreBackendModule, CyberUiFormFieldsComponent, CyberUiFormFieldsModule, CyberUiMindfullyAttendToTopicTaskProvider, CyberUiMindfullyAttendToTopicTaskProviderModule, CyberUiMinimalTaskCardComponent, CyberUiMinimalTaskCardListComponent, CyberUiMinimalTaskCardListModule, CyberUiMinimalTaskCardModule, CyberUiMinimalTaskComponent, CyberUiMinimalTaskListComponent, CyberUiMinimalTaskListModule, CyberUiMinimalTaskModule, CyberUiResponseChipsComponent, CyberUiResponseChipsModule, CyberUiSettingsDialogComponent, CyberUiSettingsDialogModule, CyberUiSettingsDialogService, CyberUiSettingsService, CyberUiSnoozeReasonCollectionDialogComponent, CyberUiSnoozeReasonCollectionDialogModule, CyberUiSnoozeReasonCollectionDialogService, CyberUiStatusIndicatorModule, CyberUiStatusIndicatorState, CyberUiTaskAccordionComponent, CyberUiTaskAccordionModule, CyberUiTasksModule, CyberUiThemeModule, CyberUiThemeService, CyberUiTimeSinceLastEngagementModule, CyberUiTimeSinceLastEngagementService, CyberUiTimeboxCountdownDisplayModule, CyberUiTimeboxModule, CyberUiTimeboxService, CyberUiTopicRegistration, CyberUiWorkOnTasksWorkflowComponent, CyberUiWorkOnTasksWorkflowModule, CyberUiWorkOnTasksWorkflowService, DelegationHistoryField, DelegationMenuComponent, DiscreteProbabilityDistributionField, ENERGY_LEVEL_FIELD, FieldListListField, FormField, FormFieldConfig, FormFieldElement, Option, TASK_SUGGESTION_SERVICE, TaskProviderRegistryService, TextField, ValueInNumericRangeField, WORKFLOW_SETTINGS_SERVICE, delegationMenuActivationHandler, getEnergyLevelField as ɵb, CyberUiChoiceFieldMenuComponent as ɵba, CyberUiFilterChipDirective as ɵbb, CyberUiFilterChipsComponent as ɵbc, MATCH_APP_AND_TOOLBAR_BACKGROUND as ɵc, TOP_TOOLBAR_DEFAULT_BACKGROUND_COLOR as ɵd, TOP_TOOLBAR_GRADIENT as ɵe, TOP_TOOLBAR_DROP_SHADOW as ɵf, CyberUiAddThingWorkflowDisplayComponent as ɵg, CyberUiAttentionalAgendaItemDisplayModule as ɵh, CyberUiActionContextDirective as ɵi, CyberUiTimerPipeModule as ɵj, TimerPipe as ɵk, CyberUiTimeboxCountdownDisplayComponent as ɵl, CyberUiAttentionalAgendaItemDisplayComponent as ɵm, CyberUiAttentionalAgendaDisplayComponent as ɵn, CyberUiFirestorePathService as ɵp, FakeModelService as ɵq, CyberUiAutofocusDirective as ɵr, CyberUiMinimalTaskExpansionPanelComponent as ɵt, CyberUiMinimalTaskExpansionPanelModule as ɵu, CyberUiSettingsSectionItemsComponent as ɵv, CyberUiStatusIndicatorComponent as ɵw, CyberUiFilterChipsModule as ɵy, CyberUiChoiceFieldMenuModule as ɵz };
//# sourceMappingURL=cyber_ui.js.map
